-- Dans la bd "crawler" ...


----------------------------------------------------------------------------------------
-- Stockage des pages deja visitees
----------------------------------------------------------------------------------------


--
-- Les sites dont on a visites toutes ou certaines pages
--

DROP TABLE Site ;
DROP SEQUENCE site_id_site_seq;
CREATE TABLE Site (
  url TEXT NOT NULL UNIQUE,
  id_site SERIAL PRIMARY KEY
);


--
-- Les pages visitee contenues dans ces sites
--

DROP TABLE Page ;
DROP SEQUENCE page_id_page_seq;
CREATE TABLE Page (
  fin_url TEXT,
  id_site INTEGER,
  date INTEGER,
  id_page SERIAL PRIMARY KEY,
  UNIQUE(fin_url,id_site),
  FOREIGN KEY  (id_site) REFERENCES Site 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);



----------------------------------------------------------------------------------------
-- Referencement des liens http
----------------------------------------------------------------------------------------


--
-- Pour stocker les liens vers des pages http
--

DROP TABLE Http ;
DROP SEQUENCE http_id_http_seq;
CREATE TABLE Http (
  lien TEXT NOT NULL UNIQUE,
  id_http SERIAL PRIMARY KEY
);


--
-- Pour stocker les mots referencant des pages html
--

DROP TABLE MotHttp ;
DROP SEQUENCE MotHttp_id_mot_seq;
CREATE TABLE MotHttp (
  mot TEXT NOT NULL UNIQUE,
  id_mot SERIAL PRIMARY KEY
);


--
-- Quel(s) lien(s) appartien(nen)t a quelle(s) page(s)
--

DROP TABLE AppHttp;
CREATE TABLE AppHttp (
  id_page INTEGER NOT NULL,
  id_http INTEGER NOT NULL,
  id_mot INTEGER NOT NULL,
  occurence INTEGER DEFAULT 1,
  UNIQUE (id_page,id_http,id_mot),
  FOREIGN KEY  (id_page) REFERENCES Page 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_mot) REFERENCES MotHttp
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_http) REFERENCES Http 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


--
-- Fonction pour inserer un lien http d'apres une chaine de caracteres.
-- Rem: Il n'y a pas de controle des acces concurents. C'est a dire, on ne verrouille
-- pas de table. Cela garantit des performance maximales (permet une concurence maximale).
-- L'integrite de la bd est assuree par les triggers que Postgresql a crees pour assures
-- l'unicite de certains champs des tables. En cas de collision entre deux processus
-- la transaction est annulee et est retentee plus tard. Ce qui suppose pour obtenir de
-- bonnes performances que ces collisions sont rares ce qui reste a demontrer mais qui
-- semble a priori probable.
-- Rem: Une collision survient quand deux processus: 
--   1)tentent de modifier en meme temps les infos liees a un meme lien http ou
--     a un meme mot (ce qui normalement mettrait la table dans un etat incoherent).
--   2)essaient d'inserer un meme lien ou un meme mot en meme temps ce qui engendrerait
--     deux entrees dans la bd pour une meme chose (ce qu'on ne veut pas).
-- Parametres:
--   $1= id de la page ds laquelle le lien a ete trouve
--   $2= url pointee par le lien
--   $3= mot descriptif du lien
--   $4= nb de fois que ce lien est designe par ce mot ds cette page
-- Retour:
--   id du lien http
--

CREATE OR REPLACE FUNCTION ins_http_url (INTEGER,TEXT,TEXT,INTEGER) RETURNS INTEGER AS
'
  DECLARE
    n_http INTEGER := -1;
    n_mot INTEGER;
    rec RECORD;
  BEGIN
    -- Recuperation de l id du lien et creation de celui-ci si necesssaire
    SELECT INTO n_http id_http FROM Http WHERE lien=$2;
    IF NOT FOUND THEN
      n_http := nextval(\'http_id_http_seq\');
      INSERT INTO Http VALUES ($2,n_http);
    END IF;
    -- Recuperation de l id du mot et creation de celui-ci si necesssaire
    SELECT INTO n_mot id_mot FROM MotHttp WHERE mot=$3;
    IF NOT FOUND THEN
      n_mot := nextval(\'mothttp_id_mot_seq\');
      INSERT INTO MotHttp VALUES ($3,n_mot);
    END IF;
    -- Regarde si ce lien est deja designe par ce mot ds cette page
    SELECT INTO rec * FROM AppHttp WHERE (id_mot=n_mot AND id_http=n_http AND id_page=$1);
    IF NOT FOUND THEN
      INSERT INTO AppHttp VALUES ($1,n_http,n_mot,$4);
    ELSIF rec.occurence <> $4 THEN
      UPDATE AppHttp SET occurence=$4 WHERE (id_mot=n_mot AND id_http=n_http AND id_page=$1);
    END IF;
    RETURN n_http;
  END;
'   
LANGUAGE  'plpgsql' ;


--
-- Fonction pour inserer un lien http d'apres son identificateur.
-- Rem: Il n'y a pas de controle des acces concurents. C'est a dire, on ne verrouille
-- pas de table. Cela garantit des performance maximales (permet une concurence maximale).
-- L'integrite de la bd est assuree par les triggers de Postgresql crees pour assures
-- l'unicite de certains champs des tables. En cas de collision entre deux processus
-- la transaction est annulee et est retentee plus tard. Ce qui suppose opur obtenir de
-- bonnes performances que ces collisions sont rares ce qui reste a demontrer mais qui
-- semble a priori probable.
-- Rem: Une collision survient quand deux processus: 
--   1)tentent de modifier en meme temps les infos liees a un meme lien http ou
--     a un meme mot (ce qui normalement mettrait la table dans un etat incoherent).
--   2)essaient d'inserer un meme lien ou un meme mot en meme temps ce qui engendrerait
--     deux entrees dans la bd pour une meme chose (ce qu'on ne veut pas).
-- Parametres:
--   $1= id de la page ds laquelle le lien a ete trouve
--   $2= id du lien
--   $3= mot descriptif du lien
--   $4= nb de fois que ce lien est designe par ce mot ds cette page
-- Retour:
--   succes= id du lien http
--

CREATE OR REPLACE FUNCTION ins_http_id (INTEGER,INTEGER,TEXT,INTEGER) RETURNS INTEGER AS
'
  DECLARE
    n_mot INTEGER;
    rec RECORD;
  BEGIN
    -- Recuperation de l id du mot et creation de celui-ci si necesssaire
    SELECT INTO n_mot id_mot FROM MotHttp WHERE mot=$3;
    IF NOT FOUND THEN
      n_mot := nextval(\'mothttp_id_mot_seq\');
      INSERT INTO MotHttp VALUES ($3,n_mot);
    END IF;
    -- Regarde si ce lien est deja designe par ce mot ds cette page
    SELECT INTO rec * FROM AppHttp WHERE (id_mot=n_mot AND id_http=$2 AND id_page=$1);
    IF NOT FOUND THEN
      INSERT INTO AppHttp VALUES ($1,$2,n_mot,$4);
    ELSIF rec.occurence <> $4 THEN
      UPDATE AppHttp SET occurence=$4 WHERE (id_mot=n_mot AND id_http=$2 AND id_page=$1);
    END IF;
    RETURN $2;
  END;
'   
LANGUAGE  'plpgsql' ;



----------------------------------------------------------------------------------------
-- Referencement des adresses mail
----------------------------------------------------------------------------------------


--
-- Pour stocker les liens vers des adresses mail
--

DROP TABLE Mail;
DROP SEQUENCE Mail_id_mail_seq;
CREATE TABLE Mail (
  mail TEXT NOT NULL UNIQUE,
  id_mail SERIAL PRIMARY KEY
);


--
-- Pour stocker les mots referencant des adresses mail
--

DROP TABLE MotMail;
DROP SEQUENCE MotMail_id_mot_seq;
CREATE TABLE MotMail (
  mot TEXT NOT NULL UNIQUE,
  id_mot SERIAL PRIMARY KEY
);


--
-- Quelle(s) adresse(s) appartien(nen)t a quelle(s) page(s) ?
--

DROP TABLE AppMail;
CREATE TABLE AppMail (
  id_page INTEGER NOT NULL,
  id_mail INTEGER NOT NULL,
  id_mot INTEGER NOT NULL,
  occurence INTEGER DEFAULT 1,
  UNIQUE (id_page,id_mail,id_mot),
  FOREIGN KEY  (id_page) REFERENCES Page 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_mot) REFERENCES MotMail
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_mail) REFERENCES Mail 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


--
-- Fonction pour inserer une adresse mail designee d'apres une chaine de caracteres.
-- Rem: Il n'y a pas de controle des acces concurents. C'est a dire, on ne verrouille
-- pas de table. Cela garantit des performance maximales (permet une concurence maximale).
-- L'integrite de la bd est assuree par les triggers que Postgresql a crees pour assures
-- l'unicite de certains champs des tables. En cas de collision entre deux processus
-- la transaction est annulee et est retentee plus tard. Ce qui suppose pour obtenir de
-- bonnes performances que ces collisions sont rares ce qui reste a demontrer mais qui
-- semble a priori probable.
-- Rem: Une collision survient quand deux processus: 
--   1)tentent de modifier en meme temps les infos liees a une meme adresse ou
--     a un meme mot (ce qui normalement mettrait la table dans un etat incoherent).
--   2)essaient d'inserer une meme adresse ou un meme mot en meme temps ce qui engendrerait
--     deux entrees dans la bd pour une meme chose (ce qu'on ne veut pas).
-- Parametres:
--   $1= id de la page ds laquelle l'adresse mail a ete trouvee
--   $2= l'adresse pointee par le lien mail
--   $3= mot descriptif du lien
--   $4= nb de fois que cette adresse est designee par ce mot ds cette page
-- Retour:
--   id de l'adresse mail
--

CREATE OR REPLACE FUNCTION ins_mail_adr (INTEGER,TEXT,TEXT,INTEGER) RETURNS INTEGER AS
'
  DECLARE
    n_mail INTEGER := -1;
    n_mot INTEGER;
    rec RECORD;
  BEGIN
    -- Recuperation de l id du lien et creation de celui-ci si necesssaire
    SELECT INTO n_mail id_mail FROM Mail WHERE mail=$2;
    IF NOT FOUND THEN
      n_mail := nextval(\'mail_id_mail_seq\');
      INSERT INTO Mail VALUES ($2,n_mail);
    END IF;
    -- Recuperation de l id du mot et creation de celui-ci si necesssaire
    SELECT INTO n_mot id_mot FROM MotMail WHERE mot=$3;
    IF NOT FOUND THEN
      n_mot := nextval(\'motmail_id_mot_seq\');
      INSERT INTO MotMail VALUES ($3,n_mot);
    END IF;
    -- Regarde si ce lien est deja designe par ce mot ds cette page
    SELECT INTO rec * FROM AppMail WHERE (id_mot=n_mot AND id_mail=n_mail AND id_page=$1);
    IF NOT FOUND THEN
      INSERT INTO AppMail VALUES ($1,n_mail,n_mot,$4);
    ELSIF rec.occurence <> $4 THEN
      UPDATE AppMail SET occurence=$4 WHERE (id_mot=n_mot AND id_mail=n_mail AND id_page=$1);
    END IF;
    RETURN n_mail;
  END;
'   
LANGUAGE  'plpgsql' ;


--
-- Fonction pour inserer une adresse mail d'apres son identificateur.
-- Rem: Il n'y a pas de controle des acces concurents. C'est a dire, on ne verrouille
-- pas de table. Cela garantit des performance maximales (permet une concurence maximale).
-- L'integrite de la bd est assuree par les triggers que Postgresql a crees pour assures
-- l'unicite de certains champs des tables. En cas de collision entre deux processus
-- la transaction est annulee et est retentee plus tard. Ce qui suppose pour obtenir de
-- bonnes performances que ces collisions sont rares ce qui reste a demontrer mais qui
-- semble a priori probable.
-- Rem: Une collision survient quand deux processus: 
--   1)tentent de modifier en meme temps les infos liees a une meme adresse ou
--     a un meme mot (ce qui normalement mettrait la table dans un etat incoherent).
--   2)essaient d'inserer une meme adresse ou un meme mot en meme temps ce qui engendrerait
--     deux entrees dans la bd pour une meme chose (ce qu'on ne veut pas).
-- Parametres:
--   $1= id de la page ds laquelle l'adresse mail a ete trouvee
--   $2= l'adresse pointee par le lien mail
--   $3= mot descriptif du lien
--   $4= nb de fois que cette adresse est designee par ce mot ds cette page
-- Retour:
--   id de l'adresse mail
--

CREATE OR REPLACE FUNCTION ins_mail_id (INTEGER,INTEGER,TEXT,INTEGER) RETURNS INTEGER AS
'
  DECLARE
    n_mot INTEGER;
    rec RECORD;
  BEGIN
    -- Recuperation de l id du mot et creation de celui-ci si necesssaire
    SELECT INTO n_mot id_mot FROM MotMail WHERE mot=$3;
    IF NOT FOUND THEN
      n_mot := nextval(\'motmail_id_mot_seq\');
      INSERT INTO MotMail VALUES ($3,n_mot);
    END IF;
    -- Regarde si ce lien est deja designe par ce mot ds cette page
    SELECT INTO rec * FROM AppMail WHERE (id_mot=n_mot AND id_mail=$2 AND id_page=$1);
    IF NOT FOUND THEN
      INSERT INTO AppMail VALUES ($1,$2,n_mot,$4);
    ELSIF rec.occurence <> $4 THEN
      UPDATE AppMail SET occurence=$4 WHERE (id_mot=n_mot AND id_mail=$2 AND id_page=$1);
    END IF;
    RETURN $2;
  END;
'   
LANGUAGE  'plpgsql' ;



----------------------------------------------------------------------------------------
-- Referencement des images
----------------------------------------------------------------------------------------


--
-- Pour stocker les liens vers des images
-- Les champs:
--  nom    : le nom de l'image sur le site
--  chemin : le chemin absolu de l'image depuis la racine du site
--  id_site: l'identificateur du site ou se trouve l'image
--  fichier: le nom du fichier contenant l'image en local 
--  id_pict: l'identificateur de l'image
-- Exemple: sur http://www.free.fr on a trouve l'image /pict/logo.gif
--          et on l'a stockee localement dans le fichier img001.gif
--  nom    : "logo.gif"
--  chemin : "/pict/"
--  id_site: 1 (si 1 est l'id de "http://www.free.fr")
--  fichier: "img001.gif"
--  id_pict: 1 (par ex)
--

DROP TABLE Pict ;
DROP SEQUENCE pict_id_pict_seq;
CREATE TABLE Pict (
  nom TEXT NOT NULL,
  chemin TEXT NOT NULL,
  id_site INTEGER,
  fichier TEXT,
  id_pict SERIAL PRIMARY KEY,
  UNIQUE(nom,chemin,id_site),
  FOREIGN KEY  (id_site) REFERENCES Site 
    ON UPDATE CASCADE
    ON DELETE CASCADE  
);


--
-- Pour stocker les mots referencant des images
--

DROP TABLE MotPict ;
DROP SEQUENCE MotPict_id_mot_seq;
CREATE TABLE MotPict (
  mot TEXT NOT NULL UNIQUE,
  id_mot SERIAL PRIMARY KEY
);


--
-- Quelle(s) image(s) appartien(nen)t a quelle(s) page(s) ?
--

DROP TABLE AppPict;
CREATE TABLE AppPict (
  id_page INTEGER NOT NULL,
  id_pict INTEGER NOT NULL,
  id_mot INTEGER NOT NULL,
  occurence INTEGER DEFAULT 1,
  UNIQUE (id_page,id_pict,id_mot),
  FOREIGN KEY  (id_page) REFERENCES Page 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_mot) REFERENCES MotPict
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_pict) REFERENCES Pict 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


--
-- Fonction pour inserer une image d'apres une chaine de caracteres.
-- Rem: Il n'y a pas de controle des acces concurents. C'est a dire, on ne verrouille
-- pas de table. Cela garantit des performance maximales (permet une concurence maximale).
-- L'integrite de la bd est assuree par les triggers que Postgresql a crees pour assures
-- l'unicite de certains champs des tables. En cas de collision entre deux processus
-- la transaction est annulee et est retentee plus tard. Ce qui suppose pour obtenir de
-- bonnes performances que ces collisions sont rares ce qui reste a demontrer mais qui
-- semble a priori probable.
-- Rem: Une collision survient quand deux processus: 
--   1)tentent de modifier en même temps les infos liees a une meme image ou
--     a un meme mot (ce qui normalement mettrait la table dans un etat incoherent).
--   2)essaient d'inserer une meme image ou un meme mot en meme temps ce qui engendrerait
--     deux entrees dans la bd pour une meme chose (ce qu'on ne veut pas).
-- Parametres:
--   $1= id du site contenant cette image
--   $2= id de la page ds laquelle un lien vers l'image a ete trouve
--   $3= nom du fichier de l'image ds le site
--   $4= chemin absolu du fichier de l'image depuis la racine du site
--   $5= fichier sys ou l'image a ete localement stockee
--   $6= mot descriptif de l'image
--   $7= nb de fois que cette image est designee par ce mot ds cette page
-- Retour:
--   id de l'image si elle n'existait pas deja avant
--   -id de l'image dans le cas contraire
--

DROP FUNCTION ins_pict_nom(INTEGER,INTEGER,TEXT,TEXT,TEXT,TEXT,INTEGER);
CREATE FUNCTION ins_pict_nom (INTEGER,INTEGER,TEXT,TEXT,TEXT,TEXT,INTEGER)
RETURNS INTEGER AS
'
  DECLARE
    new INTEGER;
    n_pict INTEGER;
    n_mot INTEGER;
    rec RECORD;
  BEGIN
    -- Recuperation de l id de l image et creation de celui-ci si necesssaire
    new := -1;
    SELECT INTO n_pict id_pict FROM Pict WHERE (id_site=$1 AND nom=$3 AND chemin=$4);
    IF NOT FOUND THEN
      new := 1;
      n_pict := nextval(\'pict_id_pict_seq\');
      INSERT INTO Pict VALUES ($3,$4,$1,$5,n_pict);
    END IF;
    -- Recuperation de l id du mot et creation de celui-ci si necesssaire
    SELECT INTO n_mot id_mot FROM MotPict WHERE mot=$6;
    IF NOT FOUND THEN
      n_mot := nextval(\'motpict_id_mot_seq\');
      INSERT INTO MotPict VALUES ($6,n_mot);
    END IF;
    -- Regarde si ce lien est deja designe par ce mot ds cette page
    SELECT INTO rec * FROM AppPict WHERE (id_mot=n_mot AND id_pict=n_pict AND id_page=$2);
    IF NOT FOUND THEN
      INSERT INTO AppPict VALUES ($2,n_pict,n_mot,$7);
    ELSIF rec.occurence <> $7 THEN
      UPDATE AppPict SET occurence=$7 WHERE (id_mot=n_mot AND id_pict=n_pict AND id_page=$2);
    END IF;
    RETURN new * n_pict;
  END;
'   
LANGUAGE  'plpgsql' ;


--
-- Fonction pour inserer une image d'apres son identificateur.
-- Rem: Il n'y a pas de controle des acces concurents. C'est a dire, on ne verrouille
-- pas de table. Cela garantit des performance maximales (permet une concurence maximale).
-- L'integrite de la bd est assuree par les triggers que Postgresql a crees pour assures
-- l'unicite de certains champs des tables. En cas de collision entre deux processus
-- la transaction est annulee et est retentee plus tard. Ce qui suppose pour obtenir de
-- bonnes performances que ces collisions sont rares ce qui reste a demontrer mais qui
-- semble a priori probable.
-- Rem: Une collision survient quand deux processus: 
--   1)tentent de modifier en même temps les infos liees a une meme image ou
--     a un meme mot (ce qui normalement mettrait la table dans un etat incoherent).
--   2)essaient d'inserer une meme image ou un meme mot en meme temps ce qui engendrerait
--     deux entrees dans la bd pour une meme chose (ce qu'on ne veut pas).
-- Parametres:
--   $1= id de la page ds laquelle un lien vers l'image a ete trouvee
--   $2= id de l'image
--   $3= mot descriptif de l'image
--   $4= nb de fois que cette image est designee par ce mot ds cette page
-- Retour:
--   id de l'image
--

CREATE OR REPLACE FUNCTION ins_pict_id (INTEGER,INTEGER,TEXT,INTEGER)
RETURNS INTEGER AS
'
  DECLARE
    n_mot INTEGER;
    rec RECORD;
  BEGIN
    -- Recuperation de l id du mot et creation de celui-ci si necesssaire
    SELECT INTO n_mot id_mot FROM MotPict WHERE mot=$3;
    IF NOT FOUND THEN
      n_mot := nextval(\'motpict_id_mot_seq\');
      INSERT INTO MotPict VALUES ($3,n_mot);
    END IF;
    -- Regarde si cette image est deja designee par ce mot ds cette page
    SELECT INTO rec * FROM AppPict WHERE (id_mot=n_mot AND id_pict=$2 AND id_page=$1);
    IF NOT FOUND THEN
      INSERT INTO AppPict VALUES ($1,$2,n_mot,$4);
    ELSIF rec.occurence <> $4 THEN
      UPDATE AppPict SET occurence=$4 WHERE (id_mot=n_mot AND id_pict=$2 AND id_page=$1);
    END IF;
    RETURN $2;
  END;
'   
LANGUAGE  'plpgsql' ;



----------------------------------------------------------------------------------------
-- Fonction de suppression
----------------------------------------------------------------------------------------


--
-- Contient le nom des fichiers de toutes les images qui ont ete supprimees.
-- Valide uniquement apres un appel a supp_page
--

DROP TABLE SuppFic;
CREATE TABLE SuppFic (
  fichier TEXT
);


--
-- Fonction pour supprimer une page.
-- Rem: On ne supprime une page que si celle-ci n'est plus accessible.
-- Parametres:
--   $1= id de la page
-- Retour:
--   n= nb de liens ou mots reellement supprimes
--

CREATE OR REPLACE FUNCTION supp_page (INTEGER) RETURNS INTEGER AS
'
  DECLARE
    id INTEGER;
    nb_supp INTEGER := 0;
    rec_lien RECORD;
    rec_mot RECORD;
    n_site INTEGER;
    fic TEXT;
  BEGIN
    -- Reinitialisation de la table SuppFic
    DELETE FROM SuppFic;
    -- Creation de tables temporaires de liens et mots a supprimer eventuellement
    CREATE LOCAL TEMP TABLE liens_http AS 
      (SELECT id_http,id_mot FROM AppHttp WHERE id_page=$1);
    CREATE LOCAL TEMP TABLE liens_mail AS 
      (SELECT id_mail,id_mot FROM AppMail WHERE id_page=$1);
    CREATE LOCAL TEMP TABLE liens_pict AS 
      (SELECT id_pict,id_mot FROM AppPict WHERE id_page=$1);
    -- Suppression des liens et des mots http inutils
    FOR rec_lien IN SELECT DISTINCT id_http FROM liens_http LOOP
        SELECT INTO id id_page FROM AppHttp
          WHERE (id_http=rec_lien.id_http AND id_page<>$1);
        IF NOT FOUND THEN
          RAISE NOTICE \'supp_http %\',rec_lien.id_http;
          DELETE FROM Http WHERE id_http=rec_lien.id_http;
          nb_supp := nb_supp + 1;
        ELSE
          RAISE NOTICE \'la page % contient le lien %\',id,rec_lien.id_http;
        END IF;
    END LOOP;
    FOR rec_lien IN SELECT DISTINCT id_mot FROM liens_http LOOP
        SELECT INTO id id_page FROM AppHttp
          WHERE (id_mot=rec_lien.id_mot AND id_page<>$1);
        IF NOT FOUND THEN
          RAISE NOTICE \'supp_mot_http %\',rec_lien.id_mot;
          DELETE FROM MotHttp WHERE id_mot=rec_lien.id_mot;
          nb_supp := nb_supp + 1;
        ELSE
          RAISE NOTICE \'la page % contient le mot %\',id,rec_lien.id_mot;
        END IF;
    END LOOP;
    -- Suppression des liens et des mots mail inutils
    FOR rec_lien IN SELECT DISTINCT id_mail FROM liens_mail LOOP
        SELECT INTO id id_page FROM AppMail
          WHERE (id_mail=rec_lien.id_mail AND id_page<>$1);
        IF NOT FOUND THEN
          RAISE NOTICE \'supp_mail %\',rec_lien.id_mail;
          DELETE FROM Mail WHERE id_mail=rec_lien.id_mail;
          nb_supp := nb_supp + 1;
        ELSE
          RAISE NOTICE \'la page % contient le lien %\',id,rec_lien.id_mail;
        END IF;
    END LOOP;
    FOR rec_lien IN SELECT DISTINCT id_mot FROM liens_mail LOOP
        SELECT INTO id id_page FROM AppMail
          WHERE (id_mot=rec_lien.id_mot AND id_page<>$1);
        IF NOT FOUND THEN
          RAISE NOTICE \'supp_mot_mail %\',rec_lien.id_mot;
          DELETE FROM MotMail WHERE id_mot=rec_lien.id_mot;
          nb_supp := nb_supp + 1;
        ELSE
          RAISE NOTICE \'la page % contient le mot %\',id,rec_lien.id_mot;
        END IF;
    END LOOP;
    -- Suppression des images, des mots et des noms d images inutils
    FOR rec_lien IN SELECT DISTINCT id_pict FROM liens_pict LOOP
        SELECT INTO id id_page FROM AppPict
          WHERE (id_pict=rec_lien.id_pict AND id_page<>$1);
        IF NOT FOUND THEN
          -- Insert le fichier de l image ds SuppFic
          SELECT INTO fic fichier FROM Pict WHERE id_pict=rec_lien.id_pict;
          INSERT INTO SuppFic VALUES (fic);
          -- Supprime l image de Pict
          DELETE FROM Pict WHERE id_pict=rec_lien.id_pict;
          RAISE NOTICE \'supp_pict %\',rec_lien.id_pict;
          nb_supp := nb_supp + 1;
        ELSE
          RAISE NOTICE \'la page % contient le lien %\',id,rec_lien.id_pict;
        END IF;
    END LOOP;
    FOR rec_lien IN SELECT DISTINCT id_mot FROM liens_pict LOOP
        SELECT INTO id id_page FROM AppPict
          WHERE (id_mot=rec_lien.id_mot AND id_page<>$1);
        IF NOT FOUND THEN
          RAISE NOTICE \'supp_mot_pict %\',rec_lien.id_mot;
          DELETE FROM MotPict WHERE id_mot=rec_lien.id_mot;
          nb_supp := nb_supp + 1;
        ELSE
          RAISE NOTICE \'la page % contient le mot %\',id,rec_lien.id_mot;
        END IF;
    END LOOP;
    -- Suppression du site si celui-ci ne contient plus de page
    SELECT INTO n_site id_site FROM Page WHERE id_page=$1;
    SELECT INTO id id_page FROM Page WHERE (id_site=n_site AND id_page<>$1);
    IF NOT FOUND THEN
      RAISE NOTICE \'supp_site %\',n_site;
      DELETE FROM Site WHERE id_site=n_site;
      nb_supp := nb_supp + 1;
    ELSE
      RAISE NOTICE \'le site % contient la page %\',n_site,id;
    END IF;
    -- Suppression de la page et de toutes les entrees comprenant cette page
    DELETE FROM Page WHERE id_page=$1;
    -- Suppression des tables temporaires
    DROP TABLE liens_http;
    DROP TABLE liens_mail;
    DROP TABLE liens_pict;
    RETURN nb_supp;
  END;
'   
LANGUAGE  'plpgsql' ;

--
-- Fonction pour supprimer un lien http.
-- Rem: Noramlement, il faudrait supprimer le mots inutilises -> a faire
-- Parametres:
--   $1= id du lien
-- Retour rien
--

CREATE OR REPLACE FUNCTION supp_lien (INTEGER) RETURNS INTEGER AS
'
  BEGIN
    -- Supprime le lien de la table Http et simultanement les entrees correspondante
    -- dans la table AppHttp
    DELETE FROM Http WHERE id_http=$1;
    RETURN 0;
  END;
'   
LANGUAGE  'plpgsql' ;
