import java.io.*;
import java.lang.*;
import java.util.*;

/**
 * Classe principale du crawler puisqu'elle contient la mthode <code>main</code>.<p>
 * Cette classe gre:
 * <ul>
 *   <li> l'initialisation de l'application (lecture du fichier de configuration, chargement des dictionnaires...)
 *   <li> le lancement du/des processus de navigation (<code>NavThread</code>) qui va/vont complter et 
 *        mettre  jour la base de donnes
 *   <li> le lancement du processus de vrification (<code>VerifThread</code>) qui va supprimer
 *        de la base de donnes toutes les pages qui n'existent plus
 *   <li> l'arrt propre de tous les processus sur demande de l'utilisateur
 * </ul>
 * @author Martin Ludovic
 * @see NavThread
 * @see VerifThread
 */
public class Crawler {
    /** Drapeau demandant au programme de signaler en sortie standard toutes les pages qu'il
     *  parse et d'autres infos */
    public static boolean verbose = true;
    /** Drapeau demandant au programme de journaliser ce qu'il fait. Si le drapeau est dsactiv
     *  seules les erreurs entrainant la fermeture d'un processus sont journalises. */
    public static boolean journal = true;
    /// Champs lis  la base de donnes
    /** Champ contenant le nom de la base de donnes dans laquelle sont stockes tous les liens trouvs sur le web. */
    public static final String name_db = "crawler";
    /** Champ contenant le nom de l'utilisateur qui doit se connecter  la base de donnes. */
    public static final String user_db = "tistou";
    /** Champ contenant le mot de passe de l'utilisateur pour se connecter  la base de donnes. */
    public static final String passwd_db = "";
    /// Champs lis au fichier de configuration
    /** Chemin par dfaut du fichier de configuration du crawler. */
    protected static final String pathConf = "../data/crawler.conf";
    /** Profondeur de recherche par dfaut utilis si le fichier de configuration n'est pas accessible. */
    protected static final String DEF_DEPTH = "0";
    /** Nombre de processus par dfaut utilis si le fichier de configuration n'est pas accessible. */
    protected static final String DEF_THREAD = "1";
    /** Page html de dpart par dfaut utilise si le fichier de configuration n'est pas accessible. */
    protected static final String DEF_START = "http://www.ufr-info-p7.jussieu.fr";
    /** Objet stockant tous les paramtres de configuration du crawler.<p>
     *  Cet objet regroupe les proprits suivantes:<ul>
     *  <li> depth      : profondeur de recherche maximale  partir d'une page de dmarrage
     *  <li> nb_threads : nombre de processus qui vont se connecter aux pages html pour les parser
     *  <li> nb_paths   : nombre de page(s) de dmarrage pour les recherches
     *  <li> path_1     : url de la 1-re page html de dmarrage
     *  <li> <i>path_i  : url de la i-me page html de dmarrage (1 < i < nb_paths+1)</i></ul> */
    protected Properties conf; 
    /// Champs des dictionnaires des mots  supprimer
    /** Chemin par dfaut des dictionnaires des mots  ignors lors du parcours des pages html. */
    protected static final String pathDicos = "../data/crawler.dico";
    /** Constante dsignant le dictionnaire franais des mots  ignors lors du parsage des pages html. */
    public static final int DICO_FR = 0;
    /** Constante dsignant le dictionnaire anglais des mots  ignors (inutilis dans cette version). */
    public static final int DICO_EN = 1;
    /** Constante dsignant le dictionnaire allemands des mots  ignors (inutilis dans cette version). */
    public static final int DICO_DE = 2;
    /** Constante stockant le nombre total de dictionnaires de mots  ignors. <p>
     *  Rem: aprs compilation il est possible de modifier le contenu des dictionnaires
     *  mais pas leur nombre. */
    protected static final int NB_DICOS = 3;
    /** Tableau contenant les dictionnairess des mots  ignors lors du parsage des pages html. */
    protected Properties[] dicos;
    /// Champs sur le fichier log
    /** Chemin par dfaut du fichier de journalisation du crawler. */
    protected static final String pathLog = "../data/crawler.log";
    /** Objet permettant d'crire dans le fichier de journalisation. */
    protected PrintStream outLog;
    /// Autres champs
    /** Tableau contenant tous les processus de navigation (<code>NavThread</code>) du crawler.
     *  @see NavThread */
    protected Vector tab_nav_procs;
    /** Objet contenant le processus de vrification (<code>VerifThread</code>) du crawler.
     *  @see VerifThread */
    protected VerifThread verif_proc;
    /** Objet contenant le processus de vrification des liens (<code>HttpVerifThread</code>) du crawler.
     *  @see HttpVerifThread */
    protected HttpVerifThread httpverif_proc;

    /**
     * Construit un nouvel objet <code>Crawler</code> qui lancera l'application.
     */
    public Crawler () {
	dicos = new Properties[NB_DICOS];
	for (int i=0 ; i<NB_DICOS ; i++) {
	    dicos[i] = new Properties();
	}
	conf = new Properties();
	tab_nav_procs = new Vector();
    }

    /**
     * Charge les paramtres de configuration du crawler par dfaut.<p>
     * Cette mthode est appele lorsque le fichier de configuration est corrompu, inexistant ou non lisible.
     * @see #DEF_DEPTH
     * @see #DEF_THREAD
     * @see #DEF_START
     */
    protected void loadDefConf () {
	// Avertissement
	System.out.println("Configuration par defaut:");
	System.out.println("  profondeur max: " + DEF_DEPTH);
	System.out.println("  nb de processus: " + DEF_THREAD);
	System.out.println("  page de depart: " + DEF_START);
	// Chargement
	conf.setProperty("depth",DEF_DEPTH);
	conf.setProperty("nb_threads",DEF_THREAD);
	conf.setProperty("nb_paths","1");
	conf.setProperty("path_1",DEF_START);
    }

    /**
     * Charge la configuration du crawler.<p>
     * Pour cela la mthode commence par essayer d'accder au fichier de configuration dont le chemin
     * est pass en paramtre. Si celui-ci est corrompu, inaccessible ou illisible, les paramtres par
     * dfaut sont simplement chargs. Si le fichier n'existe pas, l encore les paramtres par dfaut
     * sont chargs et de plus le fichier est cr avec ces mmes paramtres.
     * @param path le chemin d'accs au fichier de configuration
     * @see #loadDefConf()
     */
    protected void loadConf (String path) {
	// Ouvre le fichier
	File fic;
	try {
	    fic = new File(path);
	}
	catch (NullPointerException e) {
	    System.out.println("Valeur null pour le fichier de conf.");
	    System.out.println("Chargement des parametres par defaut.");
	    loadDefConf();
	    return;
	}
	// Verifie qu'il existe
	if (!fic.exists()) {
	    System.out.println("Le fichier " + path + " n'existe pas. Creation d'un fichier par defaut.");
	    // Chargement des parametres par defaut
	    loadDefConf();
	    try { // Tente de creer un fichier conf par defaut
		fic.createNewFile();
		// Ecriture de la conf par defaut ds un fichier
		FileOutputStream fout = new FileOutputStream(fic);
		conf.store(fout,"Configuration par defaut");
		fout.close();
	    } 
	    catch (FileNotFoundException e) {}
	    catch (IOException e) {
		System.out.println("Impossible de creer le fichier " + path + " ou d'ecrire deds !");
	    }
	}
	else if (!fic.isFile()) { // Ce n'est pas un fichier
	    System.out.println(path + " n'est pas un fichier. Chargement des parametres par defaut.");
	    // Chargement des parametres par defaut
	    loadDefConf();
	}
	else {
	    System.out.println("Chargement des parametres de " + path);
	    // Lecture du fichier
	    try {
		FileInputStream fin = new FileInputStream(fic);
		conf.load(fin);
		fin.close();
	    } 
	    catch (FileNotFoundException e) {}
	    catch (IOException e) {
		System.out.println("Impossible de lire les parametres de " + path);
		// Chargement des parametres par defaut
		loadDefConf();
	    }
	}
    }

    /**
     * Charge un dictionnaire de mots  ignors lors du parsage d'une page html.<p>
     * En cas d'erreur de lecture, le chargement est simplement abandonn et n'interrompt pas
     * l'initialisation du crawler (le fait est seulement signal sur la sortie standard).
     * @param path le chemin d'accs au fichier contenant le dictionnaire
     * @param lang la langue de ce distionnaire
     * @throws NullPointerException Emis si le paramtre <code>path</code> est <code>null</code>.
     */
    protected void loadDico (String path, int lang) throws NullPointerException {
	File fic;
	fic = new File(path);
	// Existe-t-il?
	if (fic.exists() && fic.isFile()) { // Le fichier existe et est un fichier regulier
	    // Lecture du fichier
	    try {
		FileInputStream fin = new FileInputStream(fic);
		dicos[lang].load(fin);
		System.out.println("Le dico du fichier " + path + " est charge");
		fin.close();
	    } 
	    catch (FileNotFoundException e) {}
	    catch (IOException e) {
		System.out.println("Impossible de lire le dico de " + path);
	    }
	}
	else if (!fic.exists()) {
	    System.out.println("Le fichier " + path + " n'existe pas.");
	}
	else {
	    System.out.println("Le fichier " + path + " n'est pas un fichier regulier.");
	}
    }

    /**
     * Retourne la profondeur maximale de l'exploration  partir de la page web de dpart.
     * Normalement cette profondeur est stocke dans le champ <code>Properties conf</code>. Cependant,
     * si celui-ci est corrompu, c'est la profondeur par dfaut qui est retourne.<p>
     * Rem : Ce comportement n'est pas forcment souhaitable et devrait tre modifi ultrieurement.
     * @return La profondeur maximale de l'exploration  partir de la page web de dpart.
     * @see #conf
     * @see #DEF_DEPTH
     */
    protected int getDepth() {
	Integer entier;
	try {
	    entier = new Integer(conf.getProperty("depth"));
	}
	catch (NumberFormatException e) {
	    entier = new Integer(DEF_DEPTH);
	}
	return entier.intValue();
    }

    /**
     * Retourne le nombre de processus de navigation (<code>NavThread</code>).
     * Normalement ce nombre est stock dans le champ <code>Property conf</code>. Cependant, si
     * celui-ci est corrompu, c'est le nombre par dfaut qui est retourn.<p>
     * Rem : Ce comportement n'est pas forcment souhaitable et devrait tre modifi ultrieurement.
     * @return Le nombre de processus de navigation (<code>NavThread</code>).
     * @see #conf
     * @see #DEF_THREAD
     * @see NavThread
     */
    protected int getNbThreads() {
	Integer entier;
	try {
	    entier = new Integer(conf.getProperty("nb_threads"));
	}
	catch (NumberFormatException e) {
	    entier = new Integer(DEF_THREAD);
	}
	return entier.intValue();
    }

    /**
     * Retourne le nombre de page(s) web pour commencer l'exploration.
     * Normalement ce nombre est stock dans le champ <code>Property conf</code>. Cependant, si
     * celui-ci est corrompu, c'est le nombre par dfaut (1) qui est retourn.<p>
     * Rem : Ce comportement n'est pas forcment souhaitable et devrait tre modifi ultrieurement.
     * @return Le nombre de page(s) web pour commencer l'exploration.
     * @see #conf
     */
    protected int getNbPaths() {
	Integer entier;
	try {
	    entier = new Integer(conf.getProperty("nb_paths"));
	}
	catch (NumberFormatException e) {
	    return 1;
	}
	return entier.intValue();
    }

    /**
     * Retourne l'url de la i-me page html de dmarrage des recherches.<p>
     * Cette url est stocke dans la variable <code>Properties conf</code>.
     * @return L'url de la i-me page html de dmarrage des recherche ou <code>null</code>
     *         si elle n'existe pas.
     * @see #conf
     */
    protected String getInitPath (int i) {
	String key = new String("path_" + i);
	return conf.getProperty(key);
    }

    /**
     * Ouvre un flux vers le fichier de journalisation.<p>
     * Si le fichier existe dj son contenu sera cras. Dans le cas contraire, il sera cr.
     * Si le fichier n'est pas ouvrable en criture ou n'est pas un fichier valide, une exception 
     * est lance  la fonction appelante.<p>
     * Le flux vers le fichier de journalisation est de type <code>PrintStream</code> et permet
     * donc une utilisation partage entre plusieurs processus.
     * @throws NullPointerException Emis si le chemin d'accs au fichier de journalisation n'a pas t
     *  dfini (cette exception ne devrait jamais tre lance et pourrait tre supprie ultrieurement).
     * @throws IOException Emis si le fichier de journalisation n'est pas accessible en criture.
     * @see #outLog
     */
    protected void openLog () throws NullPointerException, IOException {
	File ficLog = new File(pathLog);
	if (! ficLog.exists()) {
	    try {
		ficLog.createNewFile();
		System.out.println("Fichier log (" + pathLog + ") cree.");
	    }
	    catch (IOException e) {
		System.out.println("Impossible de creer le fichier log " + pathLog);
		throw e;
	    }
	}
	else if (! ficLog.isFile()) {
	    System.out.println("Impossible de creer le fichier log car " +pathLog+ " n'est pas un fichier.");
	    throw new IOException();
	}
	try {
	    outLog = new PrintStream(new FileOutputStream(ficLog));
	}
	catch (IOException e) {
	    System.out.println("Impossible d'ouvrir ou d'ecrire ds " + pathLog);
	    throw e;
	}
    }


    //////////////////////////////////////////////////////////////////////////////////////
    // Main
    //////////////////////////////////////////////////////////////////////////////////////

    /**
     * Point de dpart de l'application.<p>
     * Cette mthode est dcomposable en plusieurs parties et sous-parties:
     * <ul>
     *   <li> l'initialisation du crawler
     *   <ul>
     *     <li> chargement des paramtres de configuration
     *     <li> chargement des dictionnaires des mots  ignors lors du parsage des pages html
     *     <li> remplissage de la pile des pages de dmarrage des recherches
     *     <li> ouverture d'un flux vers le fichier de journalisation
     *   </ul>
     *   <li> lancement des processus qui vont alimenter et maintenir  jour la base de donne du crawler
     *   <ul>
     *     <li> le(s) processus de navigation (<code>NavThread</code>)
     *     <li> le processus de vrification (<code>VerifThread</code>)
     *   </ul>
     *   <li> coute de l'utilisateur (pour l'instant limite  savoir si il veut arrter la crawler)
     *   <li> arrt propre du programme
     *   <ul>
     *     <li> arrt des processus des processus de navigation
     *     <li> arrt du processus de vrification
     *     <li> fermeture du fichier de journalisation
     *   </ul>
     * </ul>
     * @param args liste des arguments passs  l'application.
     * @see NavThread
     * @see VerifThread
     */
    public static void main (String args[]) {
	/// Initialisation du crawler
	// Init configuration...
	Crawler crawl = new Crawler();
	crawl.loadConf(pathConf);
	// Init dico...
	String pathDico;
	for (int i=0 ; i<NB_DICOS ; i++) {
	    switch (i) {
	    case DICO_FR: 
		pathDico = pathDicos + ".fr";
		break;
	    case DICO_EN: 
		pathDico = pathDicos + ".en";
		break;
	    case DICO_DE:
		pathDico = pathDicos + ".de";
		break;
	    default:
		pathDico = pathDicos;
	    }
	    try {
		crawl.loadDico(pathDico,i);
	    }
	    catch (Exception e) {
		System.out.println("Erreur inattendue ds la methode loadDico!");
		return;
	    }
	}
	// Init pile des pages  visiter...
	WebFifo pile = null;
	try {
	    pile = new WebFifo(crawl.getDepth(),crawl.getNbThreads()*50);
	}
	catch (IllegalArgumentException e) {
	    // La profondeur de recherche max du crawler est incorrecte
	    crawl.conf.setProperty("depth","0");
	    try {
		pile = new WebFifo(0);
	    } catch (IllegalArgumentException e2) {}
	}
	String init_page;
	for (int i=1 ; i<crawl.getNbPaths()+1 ; i++) {
	    init_page = crawl.getInitPath(i);
	    if (init_page != null) {
		pile.put(new PageWeb(init_page,0)); // Les pages init ont la profondeur 0
		System.out.println("La page " + init_page + " ds la pile fifo.");
	    }
	    else
		System.out.println("La " + i + "-eme page de demarrage est null");
	}
	// Init fichier log...
	try {
	    crawl.openLog();

	    System.out.println("Fichier log pret.");
	}
	catch (NullPointerException e) {
	    System.out.println("Erreur inattendue ds la methode openLog!");
	    System.out.println("Interruption du programme!");
	    return;
	}
	catch (IOException e) {
	    System.out.println("Interruption du programme!");
	    return;
	}
	int nb_procs = crawl.getNbThreads();
	crawl.outLog.println("INIT: Lancement du crawler avec comme parametres:");
	crawl.outLog.println("INIT:   Nombre de processus: " + nb_procs);
	crawl.outLog.println("INIT:   Contenu initial de la pile des pages a visiter:");
	for (int i=1 ; i<crawl.getNbPaths()+1 ; i++) {
	    crawl.outLog.println("INIT:     " + crawl.getInitPath(i));
	}
	/// Lancement des processus
	String nom = "Proc_";
	// Processus de vrification
	crawl.verif_proc = new VerifThread(nom+"verif",crawl.outLog);
	System.out.println(nom + "verif cree.");
	try {
	    crawl.verif_proc.start();
	    System.out.println(nom + "verif lance.");
	} catch (IllegalThreadStateException e) {}
	// Processus de navigation
	NavThread nav_proc;
	for (int i=0 ; i<nb_procs ; i++) {
	    nav_proc = new NavThread(nom+"nav_"+i,crawl.outLog,
				     crawl.dicos,
				     pile,
				     crawl.verif_proc,
				     crawl.getDepth());
	    crawl.tab_nav_procs.add(nav_proc);
	    System.out.println(nom +"nav_"+ i + " cree et insere ds la pile des proc.");
	    try {
		nav_proc.start();
		System.out.println(nom +"nav_"+ i + " lance.");
	    } catch (IllegalThreadStateException e) {}
	}
	/// Contrle utilisateur
	byte b[] = new byte[1];
	int off = 0;
	b[off] = (byte)'a';
	System.out.println("Pour arreter le crawler tapez 'q'");
	while (b[off] != (byte)'q') {
	    try {
		System.in.read(b,off,1);
	    }
	    catch (IOException e) {break;}
	}
	/* A faire : dtecter qu'il n'y a plus de proc qui travaille car pile vide */
	/// Arrt propre du programme
	// Arrt des processus des processus de navigation
	for (int i=0 ; i<nb_procs ; i++) {
	    ((NavThread)crawl.tab_nav_procs.get(i)).finish();
	}
	for (int i=0 ; i<nb_procs ; i++) {
	    try {
		((NavThread)crawl.tab_nav_procs.get(i)).join();
		System.out.print("Fermeture de ");
		System.out.print(((NavThread)crawl.tab_nav_procs.get(i)).getName());
		System.out.println(" confirmee");
	    } catch (InterruptedException e) {}
	}
	// Arrt du processus de vrification
	crawl.verif_proc.finish();
	try {
	    crawl.verif_proc.join();
	    System.out.println("Fermeture de Proc_verif confirmee");
	} catch (InterruptedException e) {}
	/*
	/// Netoyage de la table http
	crawl.httpverif_proc = new HttpVerifThread(nom+"verif_lien",crawl.outLog);
	System.out.println(nom + "verif_lien cree.");
	try {
	    crawl.httpverif_proc.start();
	    System.out.println(nom + "verif_lien lance.");
	} catch (IllegalThreadStateException e) {}
	b[off] = (byte)'a';
	while (b[off] != (byte)'q') {
	    try {
		System.in.read(b,off,1);
	    }
	    catch (IOException e) {break;}
	}
	crawl.httpverif_proc.finish();
	try {
	    crawl.httpverif_proc.join();
	    System.out.println("Fermeture de Proc_verif_lien confirmee");
	} catch (InterruptedException e) {}
	*/
	// Fermer le fichier log
	crawl.outLog.close();
    }
}
