import java.util.*;

/**
 * Classe gnrale implmentant une pile FIFO.<p>
 * Un objet de ce type peut en plus tre partag entre plusieurs processus car chaque accs  la pile
 * (insertion ou suppression) est protg par un smaphore qui ne permet qu' un seul processus  la
 * fois de modifier son tat. De plus losrque la pile est vide, les processus tentant d'obtenir un
 * nouvel objet de la pile sont mis en attente jusqu' ce que la pile soit de nouveau non vide.<p>
 * Remarque: Cette classe est totalement gnrale, c'est  dire qu'elle peut tre rutilise
 * telle quelle dans une autre application ( condition toute fois d'importer galement dans cette autre
 * application la classe <code>Semaphore</code>).
 * @author Martin Ludovic
 * @see Semaphore
 */
public class LockedFifo {
    /** Taille de la pile par dfaut */
    protected static final int DEF_TAILLE = 100;
    /** Taille courante de la pile */
    protected int taille; 
    /** Taille max de la pile */
    protected int taille_max;
    /** La pile FIFO est implmente par une liste (doublement) chaine */
    protected LinkedList liste;
    /** Permet des accs concurent  la pile.<p>
     *  Ce champ est inutil car le mot clef <code>synchronized</code> joue le mme rle. Il devrait donc
     *  tre supprim ultrieurement aprs vrifications. */
    protected Semaphore sem;

    /**
     * Construit un nouvel objet <code>LockedFifo</code> ayant la taille par dfaut.<p>
     * Ce constructeur est identique  <code>LockedFifo(int)</code> avec <code>DEF_TAILLE</code>
     * comme argument.
     * @see #DEF_TAILLE
     * @see #LockedFifo(int)
     */
    public LockedFifo () {
	liste = new LinkedList();
	this.taille_max = DEF_TAILLE;
	taille = 0;
	//sem = new Semaphore();
    }

    /**
     * Construit un nouvel objet <code>LockedFifo</code> d'une capacit de <code>taille_max</code>.<p>
     * Rgle le smaphore pour ne permettre l'acces des sections critiques qu' un seul processus  la fois.
     * @param taille_max la capacit maximale de la pile FIFO
     * @throws IllegalArgumentException Si jamais la valeur passe en argument est < 1.
     */
    public LockedFifo (int taille_max) throws IllegalArgumentException {
	if (taille_max < 1)
	    throw new IllegalArgumentException("taille_max ne peut prendre de valeurs < 1.");
	liste = new LinkedList();
	this.taille_max = taille_max;
	taille = 0;
	//sem = new Semaphore();
    }

    /**
     * Retourne l'objet en bas de la pile et le supprime de celle-ci.<p>
     * Si la pile est vide la mthode met en attente tous les processus tentant de rcuprer un objet.
     * De plus en cas d'appels concurents  cette mthode ou  <code>int put(java.lang.String)</code>,
     * seul un processus  la fois accde effectivement  la pile. Cette prcaution permet de garantir
     * la cohrance de la pile car tout accs modifie celle-ci.
     * @param nom_proc Le nom du processus voulant accder  la pile (util pour connatre l'tat de
     * celui-ci).<br> Si le paramtre vaut <code>null</code> aucun message n'est affich.
     * @return L'objet en bas de la pile.
     * @see #put(java.lang.Object)
     */
    public synchronized Object get (String nom_proc) {
	Object o = null;
	// Si la pile est vide, il faut attendre qu'un processus la remplisse
	while (taille < 1) { 
	    try {
		if (nom_proc != null)
		    if (Crawler.verbose)
			System.out.println(nom_proc +" en attente devant la pile.");
		wait();
		if (nom_proc != null)
		    if (Crawler.verbose)
			System.out.println(nom_proc +" reveille pour acceder a la pile.");
	    } catch (Exception e) {}
	}
	// Section critique
	//sem.P(); // Rserve la pile
	try {
	    o = liste.getFirst(); 
	    liste.removeFirst();
	} catch (NoSuchElementException e) {/* Exception impossible */}
	taille --;
	//sem.V(); // Relche la pile
	// Retourne le 1er objet
	return o;
    }

    /**
     * Retourne l'objet en bas de la pile et le supprime de celle-ci.<p>
     * Si la pile est vide la mthode met en attente tous les processus tentant de rcuprer un objet.
     * De plus en cas d'appels concurents  cette mthode ou  <code>int put(java.lang.String)</code>,
     * seul un processus  la fois accde effectivement  la pile. Cette prcaution permet de garantir
     * la cohrance de la pile car tout accs modifie celle-ci.<p>
     * En fait cette mthode se contente d'appeler <code>get(String)</code> avec <code>null</code> en
     * argument.
     * @return L'objet en bas de la pile.
     * @see #get(java.lang.String)
     */
    public Object get () {
	return get(null);
    }

    /**
     * Insre un objet en haut de la pile.<p>
     * Si la pile est pleine la mthode ne bloque pas les processus qui voudrait rajouter un nouvel objet.
     * Cette objet est simplement ignore et le processus en est averti par une valeur de retour de 
     * <code>-1</code>. Ce comportement n'est peut-tre pas toujours souhaitable, cependant a l'est dans le
     * cas du crawler et cela est trs facilement modifiable en tendant cette classe.<p>
     * De plus en cas d'appels concurents  cette mthode ou  <code>java.lang.String get()</code>,
     * seul un processus  la fois accde effectivement  la pile. Cette prcaution permet de garantir
     * la cohrance de la pile car tout accs modifie celle-ci.
     * @param objet la nouvelle objet  rajouter  la pile
     * @return 0 si l'objet a t insre correctement, sinon -1.
     * @see #get()
     */
    public synchronized int put (Object objet) {
	// Si la pile est pleine, on ne bloque pas le processus -> jette cette objet
	if (taille > taille_max) {
	    return -1;
	}
	// Section critique
	//sem.P(); // Rserve la pile
	// Vrifie que l'objet n'est pas dj dans la pile
	if (!liste.contains(objet)) {
	    // Nouvel objet -> on la rajoute
	    liste.add(objet);
	    taille ++;
	    notify();
	}
	//sem.V(); // Relche la pile
	return 0;
    }
}
