import java.lang.*;
import java.net.*;
import java.util.*;
import java.io.*;
import java.sql.*;

/**
 * Cette classe gnre un processus qui se connecte aux pages HTML  visiter afin de les
 * analyser et de rajouter ds la base de donnes du crawler tous les liens contenus ds ces pages.<p>
 * Rem: Les processus engendrs par cette classe sont appels 'processus de navigation'.
 * @author Martin Ludovic
 */
public class NavThread extends Thread {
    /** Nombre maximum de rptitions d'une requte avant que celle-ci ne soit considre comme choue.
     *  En effet, il est possible (et normal) pour divers raisons qu'une requte choue alors
     *  qu'elle devrait russir.*/
    protected final int BOUCLE_MAX = 5;
    /** Drapeau signalant que le processus doit s'arrter. */
    protected boolean to_finish;
    /** Drapeau signalant que le processus se trouve actuellement dans un appel  la mthode
     *  <code>PageWeb getPage()</code> de la classe <code>WebFifo</code>.
     *  @see WebFifo#get() */
    protected boolean in_getPage;
    /** Tableau de dictionnaires de mots  ignorer lors du parsage.<p>
     *  Ce tableau est partag par tous les processus de navigation.
     *  Il s'agit du tableau qui a t construit dans la classe <code>Crawler</code>
     *  @see Crawler#dicos
     *  @see Crawler#loadDico(java.lang.String,int) */
    protected Properties[] dicos;
    /** Flux d'entre sur le fichier de journalisation.<p>
     *  Ce flux est partag avec les autres processus de navigation (<code>NavThread</code>)
     *  ainsi que le processus de vrification (<code>VerifThread</code>).
     *  @see VerifThread */
    protected PrintStream outLog;
    /** Pile FIFO contenant les pages  visiter.<p>
     *  Cette pile est partage par tous les processus de navigation. */
    protected WebFifo pile_pages;
    /** Objet contenant le processus de vrification du crawler.<p>
     *  Ce qui permet aux processus de navigations d'avertir ce dernier si une nouvelle page a t
     *  insre dans la base de donnes.
     *  @see VerifThread#is_new_page */
    protected VerifThread verif_proc;
    /** Objet permettant de communiquer avec la base de donnes du crawler. */
    protected Pgsql bd;
    /** Pour limiter la profondeur de recheche. */
    protected int max_depth;

    /**
     * Construit un nouvel objet <code>NavThread</code> pret  crer un nouveau processus pour
     * complter et mettre  jour la base de donnes du crawler.
     * @param nom  nom du processus
     * @param outLog flux  vers le fichier de journalisation
     * @param dicos tableau contenant tous les dictionnaires des mots  ignorer
     * @param pile_pages pile FIFO stockant les pages  visiter
     * @see #outLog
     * @see #dicos
     * @see #pile_pages
     */
    public NavThread (String nom,
		      PrintStream outLog,
		      Properties[] dicos, 
		      WebFifo pile_pages,
		      VerifThread verif_proc,
		      int max_depth) {
	super(nom);
	this.dicos = dicos;
	this.outLog = outLog;
	this.pile_pages = pile_pages;
	this.verif_proc = verif_proc;
	this.max_depth = max_depth;
	to_finish = false;
	in_getPage = false;
    }

    /**
     * Permet un arrt propre du processus.<p>
     * L'arrt n'est pas immdiat, il s'agit plutt d'une demande d'arrt permettant au processus
     * de terminer ce qu'il est en train de faire et l'empchant d'enchainer sur une nouvelle tche.
     */
    public void finish () {
	to_finish = true;
	// Verifie que le processus n'est pas bloqu dans un appel  getPage() de la pile fifo
	if (in_getPage) {
	    /* Rem: L'utilisation de la mthode <code>stop</code> est normalement  viter car elle peut
	       entrainer des verrous mortels. Ce risque est toujours possible dans notre application,
	       cependant dans ce cas trs prcis cela n'est pas drangeant. En effet, la mthode finish
	       n'est employe que lorsqu'on veut terminer l'application et qu'on veut donc terminer tous
	       les processus de navigation.
	       Cependant, il serait bien que plus tard cette terminaison soit faite de manire plus 
	       lgante en utilisant une mthode non <code>deprecated</code> comme <code>interrupt</code>. */
	    stop();
	}
    }

    /**
     * Insert dans la base de donnes le site pass en argument.
     * @param site_url l'url du site a insr dans la base de donnes
     * @return l'identificateur du site dans la base de donnes du crawler
     * @throws SQLException En cas d'erreur de communication avec la base de donnes ou de rponses
     * incohrentes de sa part.
     */
    protected int insertSiteDB (String site_url) throws SQLException {
	String query;
	int iSite;
	ResultSet rs;
	int boucle;
	boolean error,exist;
	String log;

	// Recupration d'un nouvel identificateur pour le site
	boucle = 1;
	query = "SELECT nextval('site_id_site_seq')";
	do {
	    try {
		rs = bd.querySelect(query);
		error = false;
	    }
	    catch (SQLException e) {
		//Si la requte n'a pas aboutie pour une raison quelconque, on la relance
		rs = null;
		error = true;
		boucle ++;
	    }
	} while (error && boucle<BOUCLE_MAX);
	if (boucle == BOUCLE_MAX) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
	    throw new SQLException();
	}
	try {
	    exist = rs.next();
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
	if (!exist) {
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + query + "->null => fermeture du proc");
	    throw new SQLException();
	}
	else {
	    try {
		iSite = rs.getInt(1);
	    }
	    catch (SQLException e) {
		System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
		outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
		throw e;
	    }
	}
	// Insertion de ce site ds la bd
	log = "INS_S: " +getName() + ": Insertion du site " + site_url + " ds la bd avec id_site=" + iSite;
	if (Crawler.journal)
	    outLog.println(log);
	query = "INSERT INTO Site VALUES ('" + site_url + "'," + iSite + ")";
	try {
	    bd.query(query);
	}
	catch (SQLException e) {
	    /* Il est possible qu'une erreur survienne si depuis le dernier SELECT
	       un autre processus a insere ce site */
	    // Lecture des caracteristiques de la page
	    query = "SELECT id_site FROM Site WHERE url='" + site_url + "'";
	    try {
		rs = bd.querySelect(query);
		exist = rs.next();
		if (!exist)
		    throw new SQLException();
		iSite = rs.getInt(1);
	    }
	    catch (SQLException ex) {
		System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
		outLog.println("ERROR: " + getName() + ": " + ex + " => fermeture du proc");
		throw e;
	    }
	}
	log = "INS_S_FIN: " +getName() + ": Insertion du site " + site_url;
	log += " ds la bd avec id_site=" + iSite + " terminee avec succes.";
	if (Crawler.journal)
	    outLog.println(log);

	return iSite;
    }

    /**
     * Insert dans la base de donnes la page passe en argument.<p>
     * Une page se dfinit par le site auquel elle appartient et le chemin indiquant le fichier
     * contenant son code (ce chemin peut tre vide, par exemple pour http://www.free.fr)<p>
     * La date de dernire modification est mise  0 initialement.
     * @param site_url l'url du site a insr dans la base de donnes
     * @return l'identificateur du site dans la base de donnes du crawler
     * @throws SQLException En cas d'erreur de communication avec la base de donnes ou de rponses
     * incohrentes de sa part.
     */
    protected int insertPageDB (String fin_url, int iSite) throws SQLException {
	String query;
	int iPage;
	ResultSet rs;
	int date;
	int boucle;
	boolean error,exist;
	String log;

	// Recupration d'un nouvel identificateur pour la page
	boucle = 1;
	query = "SELECT nextval('page_id_page_seq')";
	do {
	    try {
		rs = bd.querySelect(query);
		error = false;
	    }
	    catch (SQLException e) {
		//Si la requte n'a pas aboutie pour une raison quelconque, on la relance
		rs = null;
		error = true;
		boucle ++;
	    }
	} while (error && boucle<BOUCLE_MAX);
	if (boucle == BOUCLE_MAX) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
	    throw new SQLException();
	}
	try {
	    exist = rs.next();
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
	if (!exist) {
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + query + "->null => fermeture du proc");
	    throw new SQLException();
	}
	else {
	    try {
		iPage = rs.getInt(1);
	    }
	    catch (SQLException e) {
		System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
		outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
		throw e;
	    }
	}
	// Insertion de cette page ds la bd
	log = "INS_P: " +getName() + ": Insertion de la page (" + fin_url;
	log += "," + iSite + ") ds la bd avec id_page=" + iPage;
	if (Crawler.journal)
	    outLog.println(log);
	date = 0;
	query = "INSERT INTO Page VALUES ('" + fin_url + "'," + iSite + "," + date + "," + iPage + ")";
	try {
	    bd.query(query);
	}
	catch (SQLException e) {
	    /* Il est possible qu'une erreur survienne si depuis le dernier SELECT
	       un autre processus a insere cette page (Postgresql doit detecter ce doublon et 
	       interdire l'insertion pour conserver la coherrence de la bd */
	    // Lecture des caracteristiques de la page
	    query = "SELECT id_page,date FROM Page WHERE (fin_url='" + fin_url + "' AND id_site=" + iSite + ")";
	    try {
		rs = bd.querySelect(query);
		if (!rs.next())
		    throw new SQLException();
		iPage = rs.getInt(1);
		date = rs.getInt(2);
	    }
	    catch (SQLException ex) {
		System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
		outLog.println("ERROR: " + getName() + ": " + ex + " => fermeture du proc");
		throw ex;
	    }
	}
	// Avertit le processus de vrification qu'une nouvelle page a t insre
	verif_proc.newPage();
	log = "INS_P_FIN: " +getName() + ": Insertion de la page (" + fin_url;
	log += "," + iSite + ") ds la bd avec id_page=" + iPage + " terminee avec succes.";
	if (Crawler.journal)
	    outLog.println(log);

	return iPage;
    }

    /**
     * Rcupre l'identificateur d'un site dans la base de donnes.<p>
     * Si ce site n'existe pas encore dans la base de donnes, il est insr.
     * @param site_url L'url du site
     * @return <ul>
     *           <li>l'identificateur du site si tout se passe bien
     *           <li>-1 en cas d'erreur de requte
     *         </ul>
     * @see #insertSiteDB(String)
     */
    protected int getIdSite (String site_url) {
	String query;
	ResultSet rs;
	int boucle;
	int iSite;
	boolean error,exist;

	// Rcupration de l'identificateur du site
	boucle = 1;
	query = "SELECT id_site FROM Site WHERE url='" + site_url + "'";
	do {
	    try {
		rs = bd.querySelect(query);
		error = false;
	    }
	    catch (SQLException e) {
		//Si la requte n'a pas aboutie pour une raison quelconque, on la relance
		rs = null;
		error = true;
		boucle ++;
	    }
	} while (error && boucle<BOUCLE_MAX);
	if (boucle == BOUCLE_MAX) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
	    return -1;
	}
	try {
	    exist = rs.next();
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    return -1;
	}
	if (!exist) { // C'est la 1ere fois qu'une page de ce site est visite
	    // On insert la page dans la bd
	    try {
		iSite = insertSiteDB(site_url);
	    } catch (SQLException e) {return -1;}
	}
	else {
	    try {
		iSite = rs.getInt(1);
	    }
	    catch (SQLException e) {
		System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
		outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
		return -1;
	    }
	}
	return iSite;
    }

    /**
     * Recupre toutes les caractristiques d'une page (qu'on s'apprte  parser).<p>
     * Si celle-ci n'est pas encore prsente dans la base de donnes (c'est  dire si elle n'a jamais
     * t visite), elle est insre.
     * @param url l'url de la page dont on veut connatre les caractristiques
     * @return Un objet de type Page contenant les caractristiques de la page
     */
    protected Page caractPage (URL url) {
	String fin_url,site_url;
	String query;
	ResultSet rs;
	int boucle;
	int iPage,iSite;
	int date;
	boolean error,exist;
	String log;

	// Dcoupage de l'url
	//	site_url = url.getProtocol() + "://" + url.getHost() + ':' + url.getPort();
	site_url = url.getProtocol() + "://" + url.getHost();
	fin_url = url.getFile();
	if (Crawler.journal)
	    outLog.println(getName() + ": Traitement de la page " + site_url + fin_url);
	// Rcupration de l'identificateur du site
	iSite = getIdSite(site_url);
	if (iSite == -1)
	    return null;
	// Recupration des caractristiques de cette page
	boucle = 1;
	query = "SELECT id_page,date FROM Page WHERE (fin_url='" + fin_url + "' AND id_site=" + iSite + ")";
	do {
	    try {
		rs = bd.querySelect(query);
		error = false;
	    }
	    catch (SQLException e) {
		//Si la requte n'a pas aboutie pour une raison quelconque, on la relance
		rs = null;
		error = true;
		boucle ++;
	    }
	} while (error && boucle<BOUCLE_MAX);
	if (boucle == BOUCLE_MAX) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
	    return null;
	}
	try {
	    exist = rs.next();
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    return null;
	}
	if (!exist) { // C'est la 1ere fois que cette page est visite
	    // On insert la page dans la bd
	    try {
		iPage = insertPageDB(fin_url,iSite);
	    } catch (SQLException e) {return null;}
	    date = 0;
	}
	else {
	    try {
		iPage = rs.getInt(1);
		date = rs.getInt(2);
		log = getName() + ": La page " + site_url + fin_url + " existe deja (site=";
		log += iSite + ",id=" + iPage + ",date=" + date + ")";
		if (Crawler.journal)
		    outLog.println(log);
	    }
	    catch (SQLException e) {
		System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
		outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
		return null;
	    }
	}
	return new Page(fin_url,date,iPage,iSite);
    }

    /**
     * Extrait de l'url d'une page web le chemin de celle-ci, c'est  dire supprime de l'url absolue
     * le nom du fichier html contenant la page.<p>
     * Par exemple, <code>http://www.truc.fr/rep/page.htm</code> -> 
     * <code>http://www.truc.fr/rep/</code>
     * @param url l'objet reprsentant la page dont on veut connnatre la racine
     * @return La racine de la page sous forme de chaine de caractres
     */
    protected String racine(URL url) {
	String sPath;
	char[] cPath;
	int i;

	sPath = url.getPath();
	if ((sPath == null) || sPath.equals(""))
	    return  url.getProtocol() + "://" + url.getHost() + "/";
	cPath = sPath.toCharArray();
	for (i=cPath.length-1 ; i>=0 ; i--) {
	    if (cPath[i] == '/')
		return url.getProtocol() + "://" + url.getHost() + sPath.substring(0,i+1);
	}
	return  url.getProtocol() + "://" + url.getHost() + "/";
    }

    /**
     * Insre tous les liens http d'une page web dans la base de donnes du crawler.<p>
     * Cette mthode s'appuie entirement sur les fonctions <code>ins_http_url</code> et 
     * <code>ins_http_id</code> de la base de donnes.
     * @param ht_liens table de hachage contenant:
     *    <ul>
     *      <li> le nom des liens http trouvs dans la page
     *      <li> les mots rfrenant ces liens
     *      <li> le nombre de fois qu'un mot rfrence un lien
     *    </ul>
     * @param id_page l'identificateur (dans la base de donnes) de la page qui vient d'tre parse
     * @return Le nombre de liens insrs dans la base de donnes
     * @throws SQLException Emis lors d'une erreur inattendue.
     * @see "Pour plus de renseignement sur le droulement de l'insertion, voir le code des fonctions
     * <code>ins_http_url</code> et <code>ins_http_id</code> dans le fichier crawler.sql"
     */
    protected int insertHttpDB(Hashtable ht_liens, int id_page) throws SQLException {
	Hashtable ht_mots;
	Enumeration enum_liens, enum_liens_val, enum_mots, enum_mots_val;
	String lien, mot;
	Entier occ;
	int id_lien = -1;
	String query = null;
	ResultSet rs = null;
	String log;
	int boucle;
	boolean error,exist;
	int compteur = 0;

	enum_liens = ht_liens.keys();
	enum_liens_val = ht_liens.elements();
	while (enum_liens.hasMoreElements()) {
	    lien = (String)enum_liens.nextElement();
	    ht_mots = (Hashtable)enum_liens_val.nextElement();
	    enum_mots = ht_mots.keys();
	    enum_mots_val = ht_mots.elements();
	    while (enum_mots.hasMoreElements()) {
		boucle = 1;
		mot = (String)enum_mots.nextElement();
		occ = (Entier)enum_mots_val.nextElement();
		do {
		    try {
			if (id_lien < 0) {
			    query = "SELECT ins_http_url(" +id_page+ ",'" +lien+ "','" +mot+ "'," +occ.value()+ ")";
			    if (Crawler.journal)
				outLog.println("INS: " +getName()+ ": " + query);
			    rs = bd.querySelect(query);
			}
			else {
			    query = "SELECT ins_http_id(" +id_page+ "," +id_lien+ ",'" +mot+ "'," +occ.value()+ ")";
			    if (Crawler.journal)
				outLog.println("INS: " +getName()+ ": " + query);
			    rs = bd.querySelect(query);
			}
			error = false;
		    }
		    catch (SQLException e) {
			/* Il est possible que l'insertion choue dans certains cas  cause du travail en 
			   concurence de plusieurs processus NavThread. L'chec de l'insertion est voulu car 
			   il permet de conserver la cohrence de la base de donnes.
			   Dans un tel cas il suffit de recommencer l'insertion. */
			error = true;
			boucle ++;
			if (Crawler.journal)
			    outLog.println("COLLISION: " +getName()+ ": " +query+ "->echec");
		    }
		    if ((id_lien<0) && !error) {
			try {
			    rs.next();
			    id_lien = rs.getInt(1);
			}
			catch (SQLException e) {
			    // Erreur inattendue
			    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
			    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
			    throw e;
			}
		    }
		} while (error && boucle<BOUCLE_MAX);
		if (boucle == BOUCLE_MAX) {
		    /* Biein que l'erreur de ne pas pouvoir insrer un lien dans la base de donnes ne soit pas
		       trs grave en soi. Cela montre un disfonctionnement de la base de donne. Donc on arrte
		       le processus. */
		    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
		    log = "ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc";
		    outLog.println(log);
		    throw new SQLException();
		}
	    }
	    compteur ++;
	    id_lien = -1;
	}
	return compteur;
    }

    /**
     * Insert une page html dans la pile des pages  visiter.<p>
     * Quand on parse une page, tous les liens html trouvs dans cette page sont mis dans
     * la pile des pages  visiter pour tre parser  leur tour.<p>
     * Normalement, il ne faut ajouter  cette pile que des pages n'allant pas  une profondeur
     * au-del de la profondeur maximale d'exploration et n'tant pas dj prsente dans la pile.
     * Ces vrification ne sont pas faite ici, mais dans la mthode <code>put(PageWeb)</code>
     * de la classe <code>WebFifo</code>.
     * @param ht_liens tous les liens html trouvs dans la page parse
     * @param pw objet reprsentant la page parse
     * @see WebFifo#put(PageWeb)
     */
    protected void insertPagePile (Hashtable ht_liens, PageWeb pw) {
	Enumeration enum;
	String lien;

	enum = ht_liens.keys();
	while (enum.hasMoreElements()) {
	    lien = (String)enum.nextElement();
	    pile_pages.put(new PageWeb(lien,pw.depth+1));
	}
    }

    /**
     * Modifie la date de dernire modification d'une entre de la table <code>Page</code> dans 
     * la base de donnes.
     * @param id_page Identificateur de la page dont il faut modifier la date de dernire modification.
     * @param lastModifDate  La nouvelle date de dernire modification.
     * @throws SQLException En cas d'erreur de requte.
     */
    protected void setLastModif (int id_page, int lastModifDate) throws SQLException {
	String query;

	query = "UPDATE Page SET date=" + lastModifDate + " WHERE id_page=" + id_page;
	try {
	    bd.query(query);
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
    }

    /**
     * Insre tous les liens de type <code>mailto</code> d'une page web dans la base
     * de donnes du crawler.<p>
     * Cette mthode s'appuie entirement sur les fonctions <code>ins_mail_adr</code> et 
     * <code>ins_mail_id</code> de la base de donnes.
     * @param ht_liens table de hachage contenant:
     *    <ul>
     *      <li> le nom des liens mail trouvs dans la page
     *      <li> les mots rfrenant ces liens
     *      <li> le nombre de fois qu'un mot rfrence un lien
     *    </ul>
     * @param id_page l'identificateur (dans la base de donnes) de la page qui vient d'tre parse
     * @return Le nombre de liens insrs dans la base de donnes
     * @throws SQLException Emis lors d'une erreur inattendue.
     * @see "Pour plus de renseignement sur le droulement de l'insertion, voir le code des fonctions
     * <code>ins_mail_adr</code> et <code>ins_mail_id</code> dans le fichier crawler.sql"
     */
    protected int insertMailDB(Hashtable ht_liens, int id_page) throws SQLException {
	Hashtable ht_mots;
	Enumeration enum_liens, enum_liens_val, enum_mots, enum_mots_val;
	String lien, mot;
	Entier occ;
	int id_lien;
	String query = null;
	ResultSet rs = null;
	String log;
	int boucle;
	boolean error,exist;
	int compteur = 0;

	enum_liens = ht_liens.keys();
	enum_liens_val = ht_liens.elements();
	while (enum_liens.hasMoreElements()) {
	    id_lien = -1;
	    lien = (String)enum_liens.nextElement();
	    ht_mots = (Hashtable)enum_liens_val.nextElement();
	    enum_mots = ht_mots.keys();
	    enum_mots_val = ht_mots.elements();
	    while (enum_mots.hasMoreElements()) {
		boucle = 1;
		mot = (String)enum_mots.nextElement();
		occ = (Entier)enum_mots_val.nextElement();
		do {
		    try {
			if (id_lien < 0) {
			    query = "SELECT ins_mail_adr(" +id_page+ ",'" +lien+ "','" +mot+ "'," +occ.value()+ ")";
			    if (Crawler.journal)
				outLog.println("INS: " +getName()+ ": " + query);
			    rs = bd.querySelect(query);
			}
			else {
			    query = "SELECT ins_mail_id(" +id_page+ "," +id_lien+ ",'" +mot+ "'," +occ.value()+ ")";
			    if (Crawler.journal)
				outLog.println("INS: " +getName()+ ": " + query);
			    rs = bd.querySelect(query);
			}
			error = false;
		    }
		    catch (SQLException e) {
			/* Il est possible que l'insertion choue dans certains cas  cause du travail en 
			   concurence de plusieurs processus NavThread. L'chec de l'insertion est voulu car 
			   il permet de conserver la cohrence de la base de donnes.
			   Dans un tel cas il suffit de recommencer l'insertion. */
			error = true;
			boucle ++;
			if (Crawler.journal)
			    outLog.println("COLLISION: " +getName()+ ": " +query+ "->echec");
		    }
		    if ((id_lien<0) && !error) {
			try {
			    rs.next();
			    id_lien = rs.getInt(1);
			}
			catch (SQLException e) {
			    // Erreur inattendue
			    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
			    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
			    throw e;
			}
		    }
		} while (error && boucle<BOUCLE_MAX);
		if (boucle == BOUCLE_MAX) {
		    /* Biein que l'erreur de ne pas pouvoir insrer un lien dans la base de donnes ne soit pas
		       trs grave en soi. Cela montre un disfonctionnement de la base de donne. Donc on arrte
		       le processus. */
		    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
		    log = "ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc";
		    outLog.println(log);
		    throw new SQLException();
		}
	    }
	    compteur ++;
	}
	return compteur;
    }

    /**
     * Insre tous les liens de type <code>img</code> (c'est  dire vers une image) d'une page web
     * dans la base de donnes du crawler.<p>
     * L'insertion de chaque image se fait en 3 temps:
     * <ul>
     *   <li>Rapatriement de l'image dans un fichier local (appel  la mthode <code>newPictFile
     *       (String,URL)</code>).
     *   <li>Insertion de l'image dans la base de donnes (appel pour chaque mot auquel elle est
     *       associ des fonctions <code>ins_pict_nom</code> et <code>ins_pict_id</code> de la base
     *       de donnes)
     *   <li>Eventuellement suppression du fichier dans lequel on a stock l'image si celle-ci est
     *       dj stocke dans un autre fichier.
     * </ul>
     * @param ht_liens table de hachage contenant:
     *    <ul>
     *      <li> le nom des liens mail trouvs dans la page
     *      <li> les mots rfrenant ces liens
     *      <li> le nombre de fois qu'un mot rfrence un lien
     *    </ul>
     * @param page Objet contenant les caractristiques (notamment <code>id_site</code> et <code>
     * id_page</code>) de la page qui vient d'tre parse
     * @param url L'url de la page dans laquelle on a trouv l'image (ncessaire pour rcuprer celle-ci).
     * @return Le nombre de liens insrs dans la base de donnes
     * @throws SQLException Emis lors d'une erreur inattendue avec la base de donnes
     * @throws IOException Erreur de cration du fichier o on va stocker l'image ou d'criture dedans
     *         (voir la mthode <code>newPictFile(String,URL)</code>)
     * @throws SecurityException Problme de droit d'accs au fichier
     *         (voir la mthode <code>newPictFile(String,URL)</code>)
     * @throws FileNotFoundException Impossible de retrouv le fichier
     *         (voir la mthode <code>newPictFile(String,URL)</code>)
     * @see "Pour plus de renseignement sur le droulement de l'insertion, voir le code des fonctions
     * <code>ins_pict_nom</code> et <code>ins_pict_id</code> dans le fichier crawler.sql"
     * @see #newPictFile(String,URL)
     */
    protected int insertPictDB(Hashtable ht_liens, Page page, URL url) throws Exception {
	Hashtable ht_mots;
	Enumeration enum_liens, enum_liens_val, enum_mots, enum_mots_val;
	String lien,nom,chemin,mot,fichier;
	Entier occ;
	int id_lien,id_site;
	String query = null;
	ResultSet rs = null;
	String log;
	int boucle;
	boolean error,exist;
	int compteur = 0;

	enum_liens = ht_liens.keys();
	enum_liens_val = ht_liens.elements();
	while (enum_liens.hasMoreElements()) {
	    id_lien = -1;
	    ht_mots = (Hashtable)enum_liens_val.nextElement();
	    enum_mots = ht_mots.keys();
	    enum_mots_val = ht_mots.elements();
	    // Rcupre le nom et le chemin de l'image sur le serveur
	    lien = (String)enum_liens.nextElement();
	    nom = lien.substring(lien.lastIndexOf("/")+1);
	    chemin = lien.substring(0,lien.lastIndexOf("/")+1);
	    // Rcupre l'id du site
	    if (lien.startsWith("http://")) {
		// Cas particulier d'une image sur un autre site que celui de la page parse
		URL pict_url = null;
		String site;
		try {
		    pict_url = new URL(lien);
		} catch (MalformedURLException e) {continue;}
		site = "http://" + pict_url.getHost();
		id_site = getIdSite(site);
		if (id_site == -1)
		    continue;
		// Rcupre le chemin correct
		chemin = pict_url.getFile();
		chemin = chemin.substring(0,chemin.lastIndexOf("/")+1);
		// Rcupre l'image et la stocke dans un fichier local
		fichier = newPictFile(chemin+nom,pict_url);
		if (fichier == null)
		    continue;
	    }
	    else {
		// Cas classique
		id_site = page.id_site;
		// Rcupre l'image et la stocke dans un fichier local
		fichier = newPictFile(lien,url);
		if (fichier == null)
		    continue;
	    }
	    // Ajoute l'image  la bd
	    while (enum_mots.hasMoreElements()) {
		boucle = 1;
		mot = (String)enum_mots.nextElement();
		occ = (Entier)enum_mots_val.nextElement();
		do {
		    try {
			if (id_lien < 0) {
			    query = "SELECT ins_pict_nom(" + id_site + "," + page.id_page + ",'";
			    query += nom + "','" + chemin + "','" + fichier + "','" + mot + "',";
			    query += occ.value()+ ")";
			    if (Crawler.journal)
				outLog.println("INS: " +getName()+ ": " + query);
			    rs = bd.querySelect(query);
			}
			else {
			    query = "SELECT ins_pict_id(" +page.id_page+ "," +id_lien;
			    query += ",'" +mot+ "'," +occ.value()+ ")";
			    if (Crawler.journal)
				outLog.println("INS: " +getName()+ ": " + query);
			    rs = bd.querySelect(query);
			}
			error = false;
		    }
		    catch (SQLException e) {
			/* Il est possible que l'insertion choue dans certains cas  cause du travail en 
			   concurence de plusieurs processus NavThread. L'chec de l'insertion est voulu car 
			   il permet de conserver la cohrence de la base de donnes.
			   Dans un tel cas il suffit de recommencer l'insertion. */
			error = true;
			boucle ++;
			if (Crawler.journal)
			    outLog.println("COLLISION: " +getName()+ ": " +query+ "->echec");
		    }
		    if ((id_lien<0) && !error) {
			try {
			    rs.next();
			    id_lien = rs.getInt(1);
			}
			catch (SQLException e) {
			    // Erreur inattendue
			    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
			    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
			    throw e;
			}
		    }
		} while (error && boucle<BOUCLE_MAX);
		if (boucle == BOUCLE_MAX) {
		    /* Bien que l'erreur de ne pas pouvoir insrer un lien dans la base de donnes ne soit pas
		       trs grave en soi. Cela montre un disfonctionnement de la base de donne. Donc on arrte
		       le processus. */
		    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
		    log = "ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc";
		    outLog.println(log);
		    throw new SQLException();
		}
	    }
	    // Regarde si cette image n'existait pas dj dans la bd
	    if (id_lien < 0) {
		id_lien = -id_lien;
		suppPictFile(fichier);
	    }
	    compteur ++;
	}
	return compteur;
    }

    /**
     * Supprime un fichier image du systme.<p>
     * Deux raisons peuvent amener  supprimer un fichier image:
     * <ul>
     *   <li>L'image a t supprime de la base de donnes (par exemple parcequ'elle appartenait  un site
     *       qui a disparu.
     *   <li>L'image de ce fichier se trouve dj dans la base de donnes donc elle est dj stocke dans un 
     *       autre fichier.
     * </ul><p>
     * Remarque: les fichiers images se trouvent tous dans le rpertoire <code>../data/</code>.
     * @param fichier Le nom du fichier contenant une image.
     */
    protected void suppPictFile (String fichier) {
	String path;
	File fic;

	path = "../data/pict/" + fichier;
	try {
	    fic  = new File(path);
	    if (fic.isFile()) {
		fic.delete();
	    }
	}
	catch (NullPointerException e) {
	    if (Crawler.verbose)
		System.out.println("Erreur de fichier ds " + getName());
	    if (Crawler.journal)
		outLog.println("ERROR: " + getName() + ": chemin de fichier a supprimer = null");
	}
	catch (SecurityException e) {
	    if (Crawler.verbose)
		System.out.println("Erreur de fichier ds " + getName());
	    if (Crawler.journal)
		outLog.println("ERROR: " + getName() + ": impossible de supprimer " + path);
	}
    }

    /**
     * Tlcharge une image et la stocke dans un fichier dans le rpertoire <code>../data/pict
     * </code>.
     * @param path Le chemin de l'image dans le site
     * @param url L'url de la page dans laquelle on a trouv le lien (pour connatre l'url du site)
     * @return Suivant le droulement de la mthode:
     *   <ul><li>Le nom du fichier dans lequel l'image a t stocke.
     *   <li><code>null</code> en cas d'erreur non grave (le processus peut continuer)</ul>
     * @throws IOException Erreur de cration du fichier o on va stocker l'image ou d'criture dedans
     * @throws SecurityException Problme de droit d'accs au fichier
     * @throws FileNotFoundException Impossible de retrouv le fichier
     */
    protected String newPictFile (String path, URL url) throws IOException,SecurityException,FileNotFoundException {
	String log;
	int TAILLE_MAX = 512;

	/// Ouverture d'un fichier pour stocker l'image
	int index;
	File dir = null;
	File fic = null;
	String prefix,suffix,fichier,nom;
	// Initialisatioin du prefix et suffix
	prefix = "pict";
	nom = path.substring(path.lastIndexOf("/")+1);
	index = nom.lastIndexOf(".");
	if (index != -1)
	    suffix = nom.substring(index);
	else
	    suffix = "";
	// Reprage du rpertoire ccontenant toutes les images
	try {
	    dir = new File("../data/pict/");
	} catch (NullPointerException e) {}
	if (!dir.isDirectory()) {
	    System.out.println("Erreur de fichier ds " + getName() + " => fermeture du processus");
	    outLog.println("ERROR: " + getName() + " ../data n'est pas un rep => fermeture du processus");
	    throw new IOException();
	}
	// Cration d'un fichier temporaire
	try {
	    fic = File.createTempFile(prefix,suffix,dir);
	}
	catch (IllegalArgumentException e) {/* impossible */}
	catch (IOException e) {
	    System.out.println("Erreur de fichier ds " + getName() + " => fermeture du processus");
	    log = "ERROR: " + getName();
	    log += " impossible de creer un nouveau fichier pour stocker une image (" + e;
	    log += ") => fermeture du processus";
	    outLog.println(log);
	    throw e;
	}
	catch (SecurityException e) {
	    System.out.println("Erreur de fichier ds " + getName() + " => fermeture du processus");
	    log = "ERROR: " + getName();
	    log += " impossible de creer un nouveau fichier pour stocker une image (" + e;
	    log += ") => fermeture du processus";
	    outLog.println(log);
	    throw e;
	}
	// Nom du fichier cr
	fichier = fic.getName();
	/// Sauvegarde de l'image dans un fichier local
	String adr;
	URL url_pict;
	URLConnection connect;
	int cl,deplacement,octets_lus;
	InputStream is;
	byte buf[];
	FileOutputStream fout;
	int taille;
	// Reconstitution de l'url de l'image
	if (path.startsWith("http"))
	    adr = path;
	else
	    adr = url.getProtocol() + "://" + url.getHost() + path;
	try {
	    url_pict = new URL(adr);
	}
	catch (MalformedURLException e) {
	    if (Crawler.verbose)
		System.out.println("Erreur de connection ds " + getName());
	    log = "ERROR: " + getName() + adr;
	    log += " n'est pas une url valide";
	    if (Crawler.journal)
		outLog.println(log);
	    suppPictFile(fichier);
	    return null;
	}
	// Connection  l'image
	try {
	    connect = url_pict.openConnection();
	    cl = connect.getContentLength();
	    is = connect.getInputStream();
	}
	catch (IOException e) {
	    if (Crawler.verbose)
		System.out.println("Erreur de connection ds " + getName());
	    log = "ERROR: " + getName() + ": impossible de lire l'image " + adr;
	    if (Crawler.journal)
		outLog.println(log);
	    suppPictFile(fichier);
	    return null;
	}
	// Lecture de l'image
	buf = new byte[cl];
	octets_lus = 0;
	deplacement = 0;
	do {
	    if ((deplacement+TAILLE_MAX) <= cl)
		taille = TAILLE_MAX;
	    else 
		taille = cl - deplacement;
	    try {
		octets_lus = is.read(buf,deplacement,taille);
	    }
	    catch (IndexOutOfBoundsException e) {
		if (Crawler.verbose)
		    System.out.println("Erreur de telechargement ds " + getName());
		log = "ERROR: " + getName() + ": impossible de telecharger l'image " + adr;
		log += "(" + e + ")";
		if (Crawler.journal)
		    outLog.println(log);
		suppPictFile(fichier);
		return null;
	    }
	    if (octets_lus == -1) {
		// La fin du fichier a t atteinte
		break;
	    }
	    deplacement += octets_lus;
	} while (octets_lus > 0);
	if (deplacement != cl) {
	    if (Crawler.verbose)
		System.out.println("Erreur de connection ds " + getName());
	    log = "ERROR: " + getName() + ": l'image " + adr + " est probablement mal telechargee.";
	    if (Crawler.journal)
		outLog.println(log);
	    // On poursuit qd mme les oprations
	}
	// Copie de l'image dans le fichier
	try {
	    fout = new FileOutputStream(fic);
	    fout.write(buf);
	}
	catch (FileNotFoundException e) {
	    System.out.println("Erreur de fichier ds " + getName() + " => fermeture du processus");
	    log = "ERROR: " + getName();
	    log += ": impossible de trouver " + fichier;
	    log += " => fermeture du processus";
	    outLog.println(log);
	    suppPictFile(fichier);
	    throw e;
	}
	catch (SecurityException e) {
	    System.out.println("Erreur de fichier ds " + getName() + " => fermeture du processus");
	    log = "ERROR: " + getName();
	    log += ": impossible d'ouvrir un flux pour ecrire ds " + fichier;
	    log += " => fermeture du processus";
	    outLog.println(log);
	    suppPictFile(fichier);
	    throw e;
	}
	catch (IOException e) {
	    System.out.println("Erreur de fichier ds " + getName() + " => fermeture du processus");
	    log = "ERROR: " + getName();
	    log += ": impossible d'ecrire ds " + fichier;
	    log += " => fermeture du processus";
	    outLog.println(log);
	    suppPictFile(fichier);
	    throw e;
	}
	// Retourne le nom du fichier de l'image
	return fichier;
    }

    /**
     * Mthode dterminant le fonctionnement du processus.<p>
     * Ce processus permet d'alimenter la base de donnes du crawler en nouveaux liens et de mettre 
     * jour les pages ayant dj t visite.<p> 
     * Ce processus peut se dcomposer en trois tapes se rptant indfiniment tant que la mthode
     * finish() n'a pas t appele:
     * <ul>
     *   <li> rcupration d'une nouvelle page  parser
     *   <li> connection  cette page puis parsage de celle-ci
     *   <li> insertion des liens (http, mails ou images) dans la base de donnes du crawler
     * </ul>
     */
    public void run () {
	PageWeb pw; // url et profondeur de recherche de la page web
	String sPage; // url de la page web
	Page page; // caractristiques de la page web
	URL url; // adresse de la page web
	URLConnection connect; // connection  la page web
	InputStream is; // flux d'entre depuis la page web
	ParserHttp parser; // parser de page html
	Hashtable ht_pars; // liens, mails ou images retourns par le parser
	int l_date; // date de dernire modification de la page web
	int compteur; // Compte le nombre de liens insrs
	String log; // Message de journalisation


	if (Crawler.journal)
	    outLog.println("Demarrage de " + getName());
	// Connection  la base de donnes
	try {
	    bd = new Pgsql(Crawler.name_db,Crawler.user_db,Crawler.passwd_db);
	}
	catch(ClassNotFoundException e) {
	    System.out.println("Erreur de connection a la bd ds " + getName() + " -> fermeture du proc");
	    String erreur = "ERROR: " + getName();
	    erreur += ": connection bd impossible (chargement du pilote jdbc impossible): ";
	    erreur = erreur + e + " => fermeture du proc";
	    outLog.println(erreur);
	    return;
	}
	catch(SQLException sqle) {
	    System.out.println("Erreur de connection a la bd ds " + getName() + " -> fermeture du proc");
	    String erreur = "ERROR: " + getName();
	    erreur += ": connection bd impossible (acces impossible): ";
	    erreur = erreur + sqle + " => fermeture du proc";
	    outLog.println(erreur);
	    return;
	}
	while (!to_finish) { // traitement d'une page
	    /// Recupration de la page  parser
	    in_getPage = true;
	    pw = pile_pages.getPage(getName());
	    in_getPage = false;
	    sPage = pw.url;
	    if (Crawler.verbose)
		System.out.println(getName() + " parsage de " + sPage);
	    /// Parsage de la page
	    // Ouverture de la page web
	    try {
		url = new URL(sPage);
		connect = url.openConnection();
		is = connect.getInputStream();
	    } 
	    catch (MalformedURLException e) {
		if (Crawler.verbose)
		    System.out.println("Ds " +getName()+ " " +sPage+ " ne represente pas une page valide ("+e+")");
		if (Crawler.journal)
		    outLog.println("ERROR: " +getName()+ ": "+sPage+ " ne represente pas une page valide ("+e+")");
		continue;
	    }
	    catch (IOException e) {
		if (Crawler.verbose)
		    System.out.println("Ds " + getName() + " impossible d'ouvrir la page " + sPage);
		if (Crawler.journal)
		    outLog.println("ERROR: " +getName()+ ": impossible d'ouvrir la page "+sPage);
		continue;
	    }
	    // Vrification que cette page contient du code html
	    String ct = connect.getContentType();
	    if (ct == null)
		continue;
	    else if (!ct.equals("text/html"))
		continue;
	    // Rcupration des caractristiques de la page et cration si elle n'existe pas
	    if ((page = caractPage(url)) == null)
		return;
	    // Vrification si la page a t modifie depuis la dernire visite
	    l_date = (int)(connect.getLastModified()/1000);
	    if ((page.date >= l_date) && (page.date != 0)) {
		// Inutil de parser cette page elle est dj  jour
		try {
		    is.close();
		} catch (IOException ex) {}
		continue;
	    }
	    if (page.date < l_date) {
		// Il faut mettre  jour la date de dernire modif ds la bd
		try {
		    setLastModif(page.id_page,l_date);
		} catch (SQLException e) {return;}
	    }
	    // On regarde si la page n'est pas trop profonde
	    if ((max_depth != 0) && (pw.depth >= max_depth))
		continue;
	    // Cration du parseur
	    parser = new ParserHttp(racine(url),is,dicos);
	    /// Insertion des liens ds la bd
	    /* A faire dans une prochaine version:
	     * Supprimer de la bd les liens http qui ont disparus de cette page depuis la dernire
	     * visite. Cela implique:
	     * 1)Avant mise  jour de la bd, memoriser tous les liens http appartenant  la page.
	     * 2)Aprs parsage, noter tous les liens qui ont disparus.
	     * 3)Pour chacun de ces liens, le supprimer grce  une fonction de la bd suppHttp (par
	     *   ex) qui vrifierait si le lien n'appartient pas  au moins une autre page et le cas
	     *   chant le supprimerait ainsi que tous les mots de la table MotHttp qui ne seraient
	     *   lis qu' ce lien. */
	    // Traitement des liens de cette page
	    try {
		ht_pars = parser.getPages();
	    }
	    catch (IOException e) {
		// Cette erreur n'est pas critique, on passe  la page suivante
		if (Crawler.verbose)
		    System.out.println("Ds " + getName() + " impossible de parser la page " + sPage);
		if (Crawler.journal)
		    outLog.println("ERROR: " + getName() + " impossible de parser la page " + sPage);
		try {
		    is.close();
		} catch (IOException ex) {}
		continue;
	    }
	    // Insertion dans la base de donnes
	    try {
		compteur = insertHttpDB(ht_pars,page.id_page);
	    } catch (SQLException e) {return;}
	    log = "COMPT: " + getName() + " Nombre de liens http de " + sPage + " inseres ds la bd = ";
	    log += compteur;
	    if (Crawler.journal)
		outLog.println(log);
	    // Insertion dans la pile FIFO
	    insertPagePile(ht_pars,pw);
	    // Traitement des mails de cette page
	    /* A faire dans une prochaine version:
	     * Supprimer de la bd les liens mail qui ont disparus de cette page depuis la dernire
	     * visite. Cela implique:
	     * 1)Avant mise  jour de la bd, memoriser tous les liens mail appartenant  la page.
	     * 2)Aprs parsage, noter tous les liens qui ont disparus.
	     * 3)Pour chacun de ces liens, le supprimer grce  une fonction de la bd suppMail (par
	     *   ex) qui vrifierait si le lien n'appartient pas  au moins une autre page et le cas
	     *   chant le supprimerait ainsi que tous les mots de la table MotMail qui ne seraient
	     *   lis qu' ce lien. */
	    try {
		ht_pars = parser.getMails();
	    }
	    catch (IOException e) {
		// Ne peut survenir qu'au premier appel de getPages() ou getMails() ou getPicts()
	    }
	    // Insertion dans la base de donnes
	    try {
		compteur = insertMailDB(ht_pars,page.id_page);
	    } catch (SQLException e) {return;}
	    log = "COMPT: " + getName() + " Nombre de liens mail de " + sPage + " inseres ds la bd = ";
	    log += compteur;
	    if (Crawler.journal)
		outLog.println(log);
	    // Traitement des images de cette page
	    /* A faire dans une prochaine version:
	     * Supprimer de la bd les liens img qui ont disparus de cette page depuis la dernire
	     * visite. Cela implique:
	     * 1)Avant mise  jour de la bd, memoriser tous les liens img appartenant  la page.
	     * 2)Aprs parsage, noter tous les liens qui ont disparus.
	     * 3)Pour chacun de ces liens, supprimer les fichiers du rpertoire ../data/pict contenant
	     *   les images de ces liens.
	     * 4)Puis supprimer chaque lien grce  une fonction de la bd suppPict (par
	     *   ex) qui vrifierait si le lien n'appartient pas  au moins une autre page et le cas
	     *   chant le supprimerait ainsi que tous les mots de la table MotPict qui ne seraient
	     *   lis qu' ce lien. */
	    try {
		ht_pars = parser.getPicts();
	    }
	    catch (IOException e) {
		// Ne peut survenir qu'au premier appel de getPages() ou getMails() ou getPicts()
	    }
	    // Insertion dans la base de donnes
	    try {
		compteur = insertPictDB(ht_pars,page,url);
	    } catch (Exception e) {return;}
	    log = "COMPT: " + getName() + " Nombre de liens pict de " + sPage + " inseres ds la bd = ";
	    log += compteur;
	    if (Crawler.journal)
		outLog.println(log);
	    /// Fermeture de la page
	    // Fermeture du flux
	    try {
		is.close();
	    } catch (IOException e) {}
	}
	log = getName() + ": fermeture du proc (demande de l'utilisateur)";
	if (Crawler.journal)
	    outLog.println(log);
	System.out.println(log);
	bd.closeConnection();
	return;
    }
}
