import java.io.*;
import java.lang.*;
import java.sql.*;
import java.net.*;

/**
 * Cette classe gnre un processus qui pour toutes les entres de la tables <code>Page</code>
 * (c'est  dire toutes les pages dj parcourues) vrifie que la page existe encore. Dans le
 * cas contraire cette page est supprime de la base de donnes.<p>
 * Rem: La suppression d'une page entraine galement la suppression de tous les liens qu'elle
 * contenait et des mots qui dsignaient ceux-ci s'ils sont devenus inutils.<p>
 * Rem: Les processus engendrs par cette classe sont appels dans le reste de cette documentation
 * 'processus de vrification'.
 * @author Martin Ludovic
 * @see "Pour plus d'information sur le droulement d'une suppression de page, voir la fonction
 * <code>supp_page(INTEGER)</code> dans bd/crawler.sql."
 */
public class VerifThread extends Thread {
    /** Nombre maximum de tentatives de rptitions de requtes avant ne soit considre comme choue.
     *  En effet, il est possible pour divers raisons qu'une requte choue alors qu'elle devrait russir.*/
    protected final int BOUCLE_MAX = 5;
    /** Drapeau signalant que le processus doit s'arrter. */
    protected boolean to_finish;
    /** Flux d'entre sur le fichier de journalisation.
     *  Ce flux est partag avec les processus de navigation (<code>NavThread</code>)
     *  @see NavThread */
    protected PrintStream outLog;
    /** Identificateur de la page  vrifier et ventuellement  supprimer. */
    protected int id_page;
    /** Le plus grand identificateur de page utilis dans la base de donnes. */
    protected int id_max;
    /** Champ permettant de savoir si une nouvelle page  t insre dans la base de donnes
     *  par un processus de navigation. */
    protected boolean is_new_page;
    /** Drapeau indiquant si une requte <code>SELECT currval('page_id_page_seq')</code> a dj t 
     *  envoye. En effet, tant que cela n'a pas t fait on ne peut pas faire de requte <code>SELECT
     *  nextval('page_id_page_seq')</code>*/
    protected boolean first_currval;
    /** Objet permettant de communiquer avec la base de donnes du crawler. */
    protected Pgsql bd;

    /**
     * Construit un nouvel objet <code>VerifThread</code>.
     * @param nom le nom du processus
     * @param outLog le flux permettant d'crire dans le fichier de journalisation
     * @see #outLog
     */
    public VerifThread (String nom, PrintStream outLog) {
	super(nom);
	this.outLog = outLog;
	to_finish = false;
	id_page = 0;
	first_currval = true;
	is_new_page = false;
	bd = null; // L'initialisation de la bd se fait ds la mthode run
    }

    /**
     * Permet un arrt propre du processus.<p>
     * L'arrt n'est pas immdiat, il s'agit plutt d'une demande d'arrt permettant au processus
     * de terminer ce qu'il est en train de faire et l'empchant d'enchainer sur une nouvelle tche.
     * Il existe cependant un cas ou l'arrt est quasiment immdiat: celui o le processus est bloqu
     * en attente qu'au moins une page soit insre dans la base de donnes. Le processus est alors sorti
     * de sont attente et est termin.
     */
    public void finish () {
	to_finish = true;
	if (id_max == 0) {
	    try {
		interrupt();
	    } catch (SecurityException e) {}
	}
    }

    /**
     * Avertit le processus de vrification qu'une nouvelle page a t insre dans la base de donnes en
     * positionnant le drapeau <code>is_new_page</code>  <code>true</code>.<p>
     * Cette fonction n'est utilise que par les processus de navigation (<code>NavThread</code>) car ce
     * sont les seuls  pouvoir insrer quoi que ce soit dans la base de donnes.<p>
     * @see #is_new_page
     */
    public synchronized void newPage () {
	is_new_page = true;
	if (id_max == 0) {
	    try {
		notify();
	    } catch (IllegalMonitorStateException e) {}
	}
    }

    /**
     * Recherche le plus grand identificateur de page utilis dans la base de donnes.
     * @return Le plus grand identificateur de page utilis dans la base de donnes.
     * @throws SQLException En cas d'erreur engendrant l'interruption du processus
     */
    protected int maxId () throws SQLException {
	String query,erreur;
	int max;
	ResultSet rs;
	boolean exist;

	query = "SELECT max(id_page),count(id_page) FROM Page";
	try {
	    // Envoie la requte  la bd
	    rs = bd.querySelect(query);
	    // Regarde si le rsultat n'est pas vide
	    exist = rs.next();
	    if (!exist) {
		// Si la requte s'est bien passe, le rsultat contient obligatoirement 1 ligne
		throw new SQLException();
	    }
	    // Lit le rsultat
	    if (rs.getInt(2) != 0)
		max = rs.getInt(1);
	    else // aucune page n'a encore t insre dans la bd
		max = 0;
	}
	catch (SQLException e) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    erreur = "ERROR: " + getName() + ": echecs a " + query;
	    erreur += " ou lors de la lecture du rsultat => fermeture du proc";
	    outLog.println(erreur);
	    throw e;
	}
	return max;
    }

    /**
     * Recherche le nouvel identificateur maximum d'une page dans la base de donnes.<p>
     * Cette mthode permet de mettre  jour le champ <code>id_max</code>.
     * @throws SQLException En cas d'erreur de communication avec la base de donnes
     * @see #id_max
     */
    protected void newMaxId () throws SQLException {
	String query;
	ResultSet rs=null;
	boolean exist,error=true;
	int boucle;

	// Verifie qu'on peut envoyer la requte SELECT currval('page_id_page_seq')
	if (first_currval) {
	    boucle = 1;
	    query = "SELECT nextval('page_id_page_seq')";
	    do {
		try {
		    bd.querySelect(query);
		    // Le resultat de la requte ne nous interesse pas
		    error = false;
		}
		catch (SQLException e) {
		    /* Il est possible que la requte choue pour divers raisons. Dans ce cas, 
		       on la recommence jusqu' BOUCLE_MAX fois. */
		    error = true;
		    boucle++;
		}
	    } while (error && boucle<BOUCLE_MAX);
	    if (boucle == BOUCLE_MAX) {
		System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
		outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
		throw new SQLException();
	    }
	    first_currval = false;
	}
	// Envoie de la requte...
	boucle = 1;
	query = "SELECT currval('page_id_page_seq')";
	do {
	    try {
		rs = bd.querySelect(query);
		error = false;
	    }
	    catch (SQLException e) {
		/* Il est possible que la requte choue pour divers raisons. Dans ce cas, 
		   on la recommence jusqu' BOUCLE_MAX fois. */
		error = true;
		boucle++;
	    }
	} while (error && boucle<BOUCLE_MAX);
	if (boucle == BOUCLE_MAX) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
	    throw new SQLException();
	}
	// Lecture de la requte...
	try {
	    exist = rs.next();
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
	if (!exist) {
	    // La requte n'a retourne aucun rsultat -> impossible
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + query + ":aucun resultat => fermeture du proc");
	    throw new SQLException();
	}
	try {
	    // Mise  jour de id_max
	    id_max = rs.getInt(1);
	}
	catch (SQLException e) {
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
    }

    /**
     * Vrifie que l'identificateur de page pass en argument existe bien dans la base de donnes du
     * crawler, c'est  dire qu'il correspond  une page existante.
     * @param id l'identificateur de page dont on veut vrifier l'existance.
     * @return <ul>
     *           <li> <code>true</code> si l'identificateur existe.
     *           <li> <code>false</code> dans le cas contraire.
     *         </ul>
     * @throws SQLException En cas d'erreur de communication avec la base de donnes
     */
    protected boolean existId (int id) throws SQLException {
	String query;
	ResultSet rs=null;
	boolean exist,error=true;
	int boucle;

	boucle = 1;
	query = "SELECT id_page FROM Page WHERE id_page=" + id;
	do {
	    try {
		rs = bd.querySelect(query);
		error = false;
	    }
	    catch (SQLException e) {
		/* Il est possible que la requte choue pour divers raisons. Dans ce cas, 
		   on la recommence jusqu' BOUCLE_MAX fois. */
		error = true;
		boucle++;
	    }
	} while (error && boucle<BOUCLE_MAX);
	if (boucle == BOUCLE_MAX) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
	    throw new SQLException();
	}
	// Lecture de la requte...
	try {
	    exist = rs.next();
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
	if (!exist)
	    return false;
	else
	    return true;
    }

    /**
     * Retourne le plus petit identificateur de page valide dans la base de donnes du crawler.
     * @return <ul>
     *           <li>le plus petit identificateur de page valide en cas de succs.
     *           <li>-1 en cas d'chec
     *           <li>-2 si le processus a t rveill (pour terminer l'application) alors qu'il
     *               attendait qu'au moins une page soit insre dans la base de donnes.
     *         </ul>
     */
    protected synchronized int nextId () {
	int next;
	boolean error;

	// Vrifie s'il y a des pages dans la base de donnes
	if (id_max == 0) {
	    // Il n'y a aucune page dans la bd, le processus se met en attente
	    try {
		wait();
	    } catch (InterruptedException e) {return -2;}
	}
	// Vrifie si de nouvelles pages ont t rajoutes rcemment dans la bd
	if (is_new_page) {
	    // Il faut remettre  jour id_max
	    try {
		newMaxId();
		is_new_page = false;
	    } catch (SQLException e) {return -1;}
	}
	// Recherche l'identificateur de la prochaine page  vrifier
	next = id_page;
	do {
	    next++;
	    if (next > id_max)
		next = 1;	
	    // Vrifie que l'identificateur correspond  une page existante
	    try {
		error = !existId(next);
	    } catch (SQLException e) {return -1;}
	} while (error);
	return next;
    }

    /**
     * Supprime une page n'existant plus sur le net de la base de donnes du crawler.<p>
     * La page supprime est la page correspondant  l'identificateur de page courant stock
     * dans le champ <code>id_page</code>.
     * Cette mthode se droule en deux temps:
     * <ul>
     *   <li>Appel  la fonction <code>INTEGER supp_page(INTEGER)</code> de la base de donnes pour 
     *       supprimer la page ainsi que tous les liens (http, mail et img) et mots qui ne sont contenus
     *       que dans cette page. Supprime aussi le site auquel appartient la page si celui ne contient
     *       pas d'autre page.
     *   <li>Appel  la mthode <code>void suppPictFile(String)</code> pour supprimer tous les fichiers
     *       contenant des images qui ont t supprimes de la base de donnes.
     * </ul>
     * @param sPage l'url de la page  supprimer (utilis uniquement pour le fichier de journalisation).
     * @return La valeur retourne par la fonction <code>supp_page</code> de la base de donnes.
     * @throws SQLException En cas de problme de communication avec la base de donnes.
     * @see "Pour en savoir plus sur la suppression d'une page, voir la fonction supp_page
     * dans le fichier crawler.sql."
     * @see #suppPictFile(String)
     */
    protected int suppPage (String sPage) throws SQLException {
	String query;
	ResultSet rs=null;
	boolean exist,error=true;
	int boucle,res;
	String log;
	String fichier;

	/// Suppression dans la bd
	outLog.println("SUPP_DEB: " + getName() + " suppression de la page " + sPage);
	boucle = 1;
	// Appel  la fonction supp_page
	query = "SELECT supp_page(" + id_page + ")";
	do {
	    try {
		rs = bd.querySelect(query);
		error = false;
	    }
	    catch (SQLException e) {
		/* Il est possible que la requte choue pour divers raisons. Dans ce cas, 
		   on la recommence jusqu' BOUCLE_MAX fois. */
		error = true;
		boucle++;
	    }
	} while (error && boucle<BOUCLE_MAX);
	if (boucle == BOUCLE_MAX) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
	    throw new SQLException();
	}
	try {
	    exist = rs.next();
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
	if (!exist) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + query + "->aucun resultat => fermeture du proc");
	    throw new SQLException();
	}
	try {
	    res = rs.getInt(1);
	}
	catch (SQLException e) {
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
	/// Suppression dans le rep ../data/pict
	boucle = 1;
	query = "SELECT fichier FROM SuppFic";
	do {
	    try {
		rs = bd.querySelect(query);
		error = false;
	    }
	    catch (SQLException e) {
		/* Il est possible que la requte choue pour divers raisons. Dans ce cas, 
		   on la recommence jusqu' BOUCLE_MAX fois. */
		error = true;
		boucle++;
	    }
	} while (error && boucle<BOUCLE_MAX);
	if (boucle == BOUCLE_MAX) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
	    throw new SQLException();
	}
	try {
	    while (rs.next()) {
		// Rcupration d'un fichier dont l'image a t supprim de la bd
		fichier = rs.getString(1);
		// Suppression du fichier
		suppPictFile(fichier);
	    }
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
	/// Fin
	log = "SUPP_FIN: " + getName() + " suppression de la page " + sPage + " terminee avec succes (";
	log += res + " liens et/ou mots supprimes avec cette page)";
	outLog.println(log);
	return res;
    }

    /**
     * Supprime un fichier image du systme.<p>
     * Deux raisons peuvent amener  supprimer un fichier image:
     * <ul>
     *   <li>L'image a t supprime de la base de donnes (par exemple parcequ'elle appartenait  un site
     *       qui a disparu.
     *   <li>L'image de ce fichier se trouve dj dans la base de donnes donc elle est dj stocke dans un 
     *       autre fichier.
     * </ul><p>
     * Remarque: les fichiers images se trouvent tous dans le rpertoire <code>../data/</code>.
     * @param fichier Le nom du fichier contenant une image.
     */
    protected void suppPictFile (String fichier) {
	String path;
	File fic;

	path = "../data/pict/" + fichier;
	try {
	    fic  = new File(path);
	    if (fic.isFile()) {
		fic.delete();
	    }
	}
	catch (NullPointerException e) {
	    System.out.println("Erreur de fichier ds " + getName());
	    outLog.println("ERROR: " + getName() + ": chemin de fichier a supprimer = null");
	}
	catch (SecurityException e) {
	    System.out.println("Erreur de fichier ds " + getName());
	    outLog.println("ERROR: " + getName() + ": impossible de supprimer " + path);
	}
    }

    /**
     * Vrifie que la page correspondant  l'identificateur courant (<code>id_page</code>) existe encore
     * sur le net et la supprime de la base de donnes si ce n'est plus le cas.
     * @return La valeur retourne par la fonction <code>supp_page</code> de la base de donnes.
     * @throws SQLException En cas de problme de communication avec la base de donnes.
     * @see #suppPage(java.lang.String)
     * @see "Pour en savoir plus sur la suppression d'une page, voir la fonction supp_page
     * dans le fichier crawler.sql."
     */
    protected int verifPage () throws SQLException {
	String query;
	ResultSet rs=null;
	boolean exist,error=true;
	int boucle;
	String fin_url,site_url;
	URL url;
	URLConnection connect;

	// Rcupre l'url correspondant  l'identificateur de page
	boucle = 1;
	query = "SELECT url,fin_url FROM Site NATURAL JOIN Page WHERE id_page=" + id_page;
	do {
	    try {
		rs = bd.querySelect(query);
		error = false;
	    }
	    catch (SQLException e) {
		/* Il est possible que la requte choue pour divers raisons. Dans ce cas, 
		   on la recommence jusqu' BOUCLE_MAX fois. */
		error = true;
		boucle++;
	    }
	} while (error && boucle<BOUCLE_MAX);
	if (boucle == BOUCLE_MAX) {
	    System.out.println("Erreur de requete ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + BOUCLE_MAX + " echecs a " + query + " => fermeture du proc");
	    throw new SQLException();
	}
	try {
	    exist = rs.next();
	}
	catch (SQLException e) {
	    // Erreur inattendue
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
	if (!exist) {
	    /* Normalement la requte ne devrait pas renvoyer 0 rsultat car l'identificateur pass
	       en argument est sens tre valide. Cependant si ce n'est pas le cas, il ne s'agit pas
	       d'une erreur grave. On se contente donc de le signaler  la mthode appelante mais pas
	        l'utilisateur. */
	    return -1;
	}
	try {
	    site_url = rs.getString(1);
	    fin_url = rs.getString(2);
	}
	catch (SQLException e) {
	    System.out.println("Erreur inattendue ds " + getName() + " -> fermeture du proc");
	    outLog.println("ERROR: " + getName() + ": " + e + " => fermeture du proc");
	    throw e;
	}
	// Vrifie que cette page existe encore sur le net
	try {
	    url = new URL(site_url+fin_url);
	    connect = url.openConnection();
	} 
	catch (Exception e) {
	    // Si une erreur survient, cela signifie que la page n'existe plus
	    // Suppression de la page dans la base de donnes du crawler
	    try {
		return suppPage(site_url+fin_url);
	    } catch (SQLException ex) {throw ex;}
	}
	return 0;
    }

    /**
     * Mthode dterminant le fonctionnement du processus.<p>
     * La majorit des champs de l'objet sont initialiss dans le constructeur, cependant certains le sont
     * dans cette mthode:
     * <ul>
     *   <li>la connection  la base de donnes (<code>bd</code>)
     *   <li>le plus grand identificateur de page utilis dans la base de donnes (<code>id_max</code>)
     * </ul>
     * La tche de ce processus se dcompose en trois tapes:
     * <ul>
     *   <li>Recupration de l'identificateur d'une page  parser
     *   <li>Vrification si la page existe toujours sur le net
     *   <li>Suppression de la page si elle n'existe plus
     * </ul>
     */
    public void run () {
	Page page;
	String erreur,log;

	outLog.println("Demarrage de " + getName());
	// Connection  la base de donnes
	try {
	    bd = new Pgsql(Crawler.name_db,Crawler.user_db,Crawler.passwd_db);
	}
	catch(ClassNotFoundException e) {
	    System.out.println("Erreur de connection a la bd ds " + getName() + " -> fermeture du proc");
	    erreur = "ERROR: " + getName();
	    erreur += ": connection bd impossible (chargement du pilote jdbc impossible): ";
	    erreur = erreur + e + " => fermeture du proc";
	    outLog.println(erreur);
	    return;
	}
	catch(SQLException sqle) {
	    System.out.println("Erreur de connection a la bd ds " + getName() + " -> fermeture du proc");
	    erreur = "ERROR: " + getName();
	    erreur += ": connection bd impossible (acces impossible): ";
	    erreur = erreur + sqle + " => fermeture du proc";
	    outLog.println(erreur);
	    return;
	}
	// Rcupre le plus grand identificateur de page utilis dans la base de donnes
	try {
	    id_max = maxId();
	}
	catch (SQLException e) {
	    return;
	}
	while (!to_finish) { // traitement d'une page
	    // Recupration d'une page  parser
	    id_page = nextId();
	    if (id_page < 0)
		return;
	    // Suppression ventuelle de la page
	    try {
		verifPage();
	    } catch (SQLException e) {return;}
	}
	log = getName() + ": fermeture du proc (demande de l'utilisateur)";
	outLog.println(log);
	System.out.println(log);
	bd.closeConnection();
	return;
    }
}
