import javax.swing.text.html.parser.*;
import java.lang.*;
import java.util.*;
import java.io.*;

/**
 * Cette classe fournit aux processus de navigation (<code>NavThread</code>) des mthodes pour parser
 * les pages qu'ils vont rencontrer afin d'en extraire les liens de type :
 * <ul>
 *   <li><code>http</code> (<code><A HREF="http://www.ufr-info-p7.jussieu.fr/">University of Paris 7</A>
 *       </code>)
 *   <li><code>mailto</code> (<code><A HREF="mailto:Mihaela.Sighireanu@liafa.jussieu.fr">Mihaela Sighireanu</A>
 *       </code>)
 *   <li><code>img</code> (<code><IMG SRC="ICONS/redball_light.gif" HEIGHT=13 WIDTH=13 ALT="REDBALL"></code>)
 * </ul>
 * @author Martin Ludovic
 */
public class ParserHttp {
    /** Tableau des dictionnaires des mots  ignors lors du parsage. */
    protected Properties[] dicos;
    /** Flux permettant de lire le contenu de la page.*/
    protected InputStream is;
    /** Le chemin du rpertoire contenant le fichier source de la page  parser.<p>
     *  Par exemple si la page  parser est <code>http://www.server.fr/rep1/rep2/fic.html</code> alors
     *  <code>root</code> contiendra <code>http://www.server.fr/rep1/rep2/</code>.<p>
     *  Ce champ est ncessaire pour transformer les chemins relatifs rencontrs dans cette page
     *  en chemin absolus. */
    protected String root;
    /** Drapeau indiquant si la page a t parse ou non.
     *  <ul><li><code>true</code> si la page a dj t parse.
     *  <li><code>false</code> sinon.</ul> */
    protected boolean isPars;
    /** Contient aprs parsage tous les liens de type <code>http</code> contenus dans cette page.*/
    protected Hashtable pages_ht;
    /** Contient aprs parsage tous les liens de type <code>mailto</code> contenus dans cette page.*/
    protected Hashtable mails_ht;
    /** Contient aprs parsage tous les liens de type <code>img</code> contenus dans cette page.*/
    protected Hashtable picts_ht;

    /**
     * Construit un nouvel objet de type <code>ParserHttp</code>.
     * @param root Le chemin du rpertoire contenant le fichier source de la page  parser
     * @param is Un flux sur le contenu de la page  parser.
     * @param dicos Tableau des dictionnaires des mots  ne pas prendre en compte
     * @see #root
     * @see #is
     * @see #dicos
     * @see NavThread#racine(URL)
     * @see Crawler#dicos
     */
    public ParserHttp (String root, InputStream is, Properties[] dicos) {
	this.dicos = dicos;
	this.is = is;
	this.root = root;
	isPars = false;
	pages_ht = new Properties();
	mails_ht = new Properties();
	picts_ht = new Properties();
    }

    /**
     * Retourne la table de hachage contenant tous les liens <code>http</code> contenus dans cette
     * page.<p>
     * Cependant, si un lien est non valide (par ex un lien vers la page <code>http://www.somewhere.com/
     * rep1/../../trux.html"</code>), il n'est pas insr dans cette table.
     * @return La table de hachage contenant les liens <code>http</code>.
     */
    public Hashtable getPages () throws IOException {
	// Vrifie si la page a dj t parse
	if (isPars)
	    return pages_ht;
	parse();
	return pages_ht;
    }

    /**
     * Retourne la table de hachage contenant tous les liens <code>mailto</code> contenus dans cette
     * page.<p>
     * @return La table de hachage contenant les liens <code>mailto</code>.
     */
    public Hashtable getMails ()  throws IOException {
	// Vrifie si la page a dj t parse
	if (isPars)
	    return mails_ht;
	parse();
	return mails_ht;
    }
 
    /**
     * Retourne la table de hachage contenant tous les liens <code>img</code> contenus dans cette
     * page.<p>
     * Cependant, si un lien est non valide (par ex un lien vers l'image <code>/rep1/../../trux.gif"</code>),
     * il n'est pas insr dans cette table.
     * @return La table de hachage contenant les liens <code>img</code>.
     */
    public Hashtable getPicts () throws IOException {
	// Vrifie si la page a dj t parse
	if (isPars)
	    return picts_ht;
	parse();
	return picts_ht;
    }

    /**
     * Parse la page html passe au constructeur de cet objet.<p>
     * Le parsage consiste  remplir trois tables de hachage avec les liens trouvs dans la page.<br>
     * Ces liens tant de type <code>http</code>, <code>mailto</code> et <code>img</code>.<p>
     * Le parsage se base sur les mthodes de la classe <code>CrawlerCallbackHandler</code>.
     * @see CrawlerCallbackHandler
     */
    protected void parse () throws IOException {
	CrawlerCallbackHandler handler;
	Hashtable ht_tab[];
	BufferedReader rdr;

	// Prparation du handler
	ht_tab = new Hashtable[3];
	ht_tab[0] = pages_ht;
	ht_tab[1] = mails_ht;
	ht_tab[2] = picts_ht;
	handler = new CrawlerCallbackHandler(ht_tab,dicos,root);
	// Parsage
        rdr = new BufferedReader(new InputStreamReader(is)); 
	new ParserDelegator().parse(rdr, handler, true);
	// Indique que la page a t parse
	isPars = true;
    }

}


