-- Dans la bd "moteur_web" ...


--
-- Les serveurs sur lesquels on peut touver des pages web, des images, des archives et des documents texte
--

DROP TABLE Serveur ;
DROP SEQUENCE serveur_id_serveur_seq;
CREATE TABLE Serveur (
  url_serveur TEXT NOT NULL UNIQUE,
  id_serveur SERIAL PRIMARY KEY
);


----------------------------------------------------------------------------------------
-- Tables concernant les objets stockes sur un serveur :
-- les pages web, les images et les autres ficiers (y compris les documents texte).
----------------------------------------------------------------------------------------


-------------------------------------------
-- Les images (.jpg, .gif, .bmp, ...)
--

DROP TABLE Img ;
DROP SEQUENCE img_id_img_seq;
CREATE TABLE Img (
  id_serveur INTEGER,
  path TEXT NOT NULL,
  nom TEXT NOT NULL,
  id_img SERIAL PRIMARY KEY,
  UNIQUE(id_serveur,path,nom),
  FOREIGN KEY  (id_serveur) REFERENCES Serveur 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-------------------------------------------
-- Les pages web (.htm,.html, .php, ...)
--

DROP TABLE Page ;
DROP SEQUENCE page_id_page_seq;
CREATE TABLE Page (
  id_serveur INTEGER,
  url_page TEXT NOT NULL,
  date INTEGER DEFAULT 0,
  id_page SERIAL PRIMARY KEY,
  UNIQUE(url_page,id_serveur),
  FOREIGN KEY  (id_serveur) REFERENCES Serveur 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-------------------------------------------
-- Les autres fichiers (y compris les documents texte)
--

-- L'entite 'fichier'

DROP TABLE Fichier ;
DROP SEQUENCE fichier_id_fichier_seq;
CREATE TABLE Fichier (
  path TEXT NOT NULL,
  nom TEXT NOT NULL,
  id_fichier SERIAL PRIMARY KEY,
  UNIQUE(path,nom)
);

-- L'entite 'documents texte' (.doc, .ps, .pdf, .txt, ...)

DROP TABLE Doc_Txt ;
CREATE TABLE Doc_Txt (
  id_fichier INTEGER PRIMARY KEY,
  FOREIGN KEY  (id_fichier) REFERENCES Fichier 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-- L'entite 'protocole'

DROP TABLE Protocole ;
DROP SEQUENCE protocole_id_proto_seq;
CREATE TABLE Protocole (
  prefixe TEXT NOT NULL UNIQUE,
  id_proto SERIAL PRIMARY KEY
);

-- L'association entre les entites 'fichier', 'seveur' et 'protocole'
-- Pour connaitre quel protocole utilise pour acceder au fichier x sur le serveur y.

DROP TABLE Heberge;
CREATE TABLE Heberge (
  id_fichier INTEGER,
  id_proto INTEGER,
  id_serveur INTEGER,
  UNIQUE (id_fichier,id_proto,id_serveur),
  FOREIGN KEY  (id_fichier) REFERENCES Fichier 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_proto) REFERENCES Protocole 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_serveur) REFERENCES Serveur 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);



----------------------------------------------------------------------------------------
-- Tables concernant le contenu informatif (texte brut) des documents texte et des pages web
----------------------------------------------------------------------------------------


--
-- Pour stocker tous les mots
--

DROP TABLE Mot_Txt ;
DROP SEQUENCE mot_txt_id_txt_seq;
CREATE TABLE Mot_Txt (
  mot TEXT NOT NULL UNIQUE,
  id_txt SERIAL PRIMARY KEY
);


--
-- Pour savoir quel mot est contenu dans quel document texte
--

DROP TABLE Txt_doc;
CREATE TABLE Txt_doc (
  id_txt INTEGER,
  id_fichier INTEGER,
  occurrence INTEGER DEFAULT 1,
  UNIQUE (id_fichier,id_txt),
  FOREIGN KEY  (id_fichier) REFERENCES Doc_txt 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_txt) REFERENCES Mot_txt 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


--
-- Pour savoir quel mot est contenu dans quelle page web
--

DROP TABLE Txt_page;
CREATE TABLE Txt_page (
  id_txt INTEGER,
  id_page INTEGER,
  occurrence INTEGER DEFAULT 1,
  UNIQUE (id_page,id_txt),
  FOREIGN KEY  (id_page) REFERENCES Page 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_txt) REFERENCES Mot_txt 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);



----------------------------------------------------------------------------------------
-- Tables concernant les elements d'une page web (autre que le texte brut) :
-- liens hyper-texte, adresses electroniques et images.
----------------------------------------------------------------------------------------


-------------------------------------------
-- Pour stocker les mots decrivant ces elements
--

DROP TABLE Mot_Desc ;
DROP SEQUENCE mot_desc_id_desc_seq;
CREATE TABLE Mot_Desc (
  mot TEXT NOT NULL UNIQUE,
  id_desc SERIAL PRIMARY KEY
);


-------------------------------------------
-- Les liens hyper-texte
--

-- L'entite 'Lien hyper-texte'

DROP TABLE Href ;
DROP SEQUENCE href_id_href_seq;
CREATE TABLE Href (
  type_fic INTEGER NOT NULL,
  id_fic INTEGER NOT NULL,
  id_href SERIAL PRIMARY KEY,
  UNIQUE (type_fic,id_fic)
);

-- Verifie que l'attribut id_fic correspond a un fichier existant si type_fic = 1
-- ou a une page existante si type_fic = 2

DROP FUNCTION check_id_proc();
CREATE FUNCTION check_id_proc() RETURNS opaque 
AS '
  DECLARE
    reponse INTEGER;
  BEGIN
    --RAISE NOTICE \'    debut position_perso_proc\';
    IF NEW.type_fic = 1 THEN -- Les fichiers
      SELECT id_fichier INTO reponse FROM Fichier WHERE id_fichier = NEW.id_fic;
      IF reponse IS NULL THEN
        --RAISE NOTICE \'    fin check_id_proc: echec\';
        RETURN NULL;
      END IF;
    ELSE
      IF NEW.type_fic = 2 THEN -- Les pages web
        SELECT id_page INTO reponse FROM Page WHERE id_page = NEW.id_fic;
        IF reponse IS NULL THEN
          --RAISE NOTICE \'    fin check_id_proc: echec\';
          RETURN NULL;
        END IF;
      ELSE
        --RAISE NOTICE \'    fin check_id_proc: echec\';
        RETURN NULL;
      END IF;
    END IF;
    --RAISE NOTICE \'    fin check_id_proc: reussite\';
    RETURN NEW;
  END;
'
LANGUAGE 'plpgsql';

CREATE TRIGGER check_id_trig
  BEFORE INSERT ON Href
  FOR EACH ROW
    EXECUTE PROCEDURE check_id_proc();


-- L'association entre 'Lien hyper-texte', 'Page web' et 'Mot descrptif'
-- Pour savoir quel(s) lien(s) appartien(nen)t a quelle(s) page(s) et 
-- par quel(s) mots il(s) est(sont) decrit(s).

DROP TABLE App_href;
CREATE TABLE App_href (
  id_page INTEGER,
  id_href INTEGER,
  id_desc INTEGER,
  occurrence INTEGER DEFAULT 1,
  UNIQUE (id_page,id_href,id_desc),
  FOREIGN KEY  (id_page) REFERENCES Page 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_desc) REFERENCES Mot_desc
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_href) REFERENCES Href 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-------------------------------------------
-- Les adresses electroniques
--

-- L'entite 'Adresse electronique'

DROP TABLE Mail ;
DROP SEQUENCE mail_id_mail_seq;
CREATE TABLE Mail (
  mail TEXT NOT NULL UNIQUE,
  id_mail SERIAL PRIMARY KEY
);


-- L'association entre 'Adresse electronique', 'Page web' et 'Mot descrptif'
-- Pour savoir quel(s) email(s) appartien(nen)t a quelle(s) page(s) et 
-- par quel(s) mots il(s) est(sont) decrit(s).

DROP TABLE App_mail;
CREATE TABLE App_mail (
  id_page INTEGER,
  id_mail INTEGER,
  id_desc INTEGER,
  occurrence INTEGER DEFAULT 1,
  UNIQUE (id_page,id_mail,id_desc),
  FOREIGN KEY  (id_page) REFERENCES Page 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_desc) REFERENCES Mot_desc
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_mail) REFERENCES Mail 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);


-------------------------------------------
-- Les images
--

-- L'association entre 'Image', 'Page web' et 'Mot descrptif'
-- Pour savoir quelle(s) images(s) appartien(nen)t a quelle(s) page(s) et 
-- par quel(s) mots elle(s) est(sont) decrite(s).

DROP TABLE App_img;
CREATE TABLE App_img (
  id_page INTEGER,
  id_img INTEGER,
  id_desc INTEGER,
  occurrence INTEGER DEFAULT 1,
  UNIQUE (id_page,id_img,id_desc),
  FOREIGN KEY  (id_page) REFERENCES Page 
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_desc) REFERENCES Mot_desc
    ON UPDATE CASCADE
    ON DELETE CASCADE,
  FOREIGN KEY  (id_img) REFERENCES Img 
    ON UPDATE CASCADE
    ON DELETE CASCADE
);
